/***************************************************************************
 *                                                                         *
 *   src/cad/src/engine/cadarc.cpp                                         *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cad/engine/cadarc.h>

#include <cmath>

using namespace cad::engine;

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

CADArc::CADArc() : CADEntity("0", CADPen())
{
	radius = 0.0;
	startAngle = 0.0;
	endAngle = 0.0;
}

CADArc::CADArc(const CADVector& center, double radius, double startAngle, double endAngle,
	QString layer, CADPen pen) : CADEntity(layer, pen), center(center), radius(radius),
	startAngle(startAngle), endAngle(endAngle)
{
	this->center = center;
	this->radius = radius;
	this->startAngle = startAngle;
	this->endAngle = endAngle;
}

CADArc::~CADArc()
{

}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

CADVector CADArc::getCenter() const
{
	return center;
}

double CADArc::getCenterX() const
{
	return center.getX();
}

double CADArc::getCenterY() const
{
	return center.getY();
}

double CADArc::getRadius() const
{
	return radius;
}

double CADArc::getStartAngle() const
{
	return startAngle;
}

double CADArc::getEndAngle() const
{
	return endAngle;
}

double CADArc::getAngle() const
{
	return fabs(endAngle - startAngle);
}

double CADArc::getLength() const
{
	double angle = endAngle - startAngle;
	return fabs((radius * (4*atan(1.0)) * angle) / 180);
}

CADVector CADArc::getStartPoint() const
{
	CADVector rel;
	rel.setPolar(radius, startAngle);
	return center + rel;
}

CADVector CADArc::getEndPoint() const
{
	CADVector rel;
	rel.setPolar(radius, endAngle);
	return center + rel;
}

void CADArc::setCenter(const CADVector& center)
{
	this->center = center;
}

void CADArc::setCenterX(double x)
{
	center.setX(x);
}

void CADArc::setCenterY(double y)
{
	center.setY(y);
}

void CADArc::setCenter(double x, double y)
{
	center.set(x, y);
}

void CADArc::setRadius(double radius)
{
	this->radius = radius;
}

void CADArc::setStartAngle(double startAngle)
{
	this->startAngle = startAngle;
}

void CADArc::setEndAngle(double endAngle)
{
	this->endAngle = endAngle;
}

/**
 * set the included angle
 * @param startPointFixed: if true, the angle is added to the
 * start angle, else the angle is subtracted from the end angle
 */
void CADArc::setAngle(double angle, bool startAngleFixed)
{
	if (startAngleFixed)
		endAngle = startAngle + angle;
	else
		startAngle = endAngle - angle;
}

/**
 * define the arc by 3 points
 */
void CADArc::set3Points(CADVector start, CADVector second, CADVector end)
{
	CADVector a = start - second; // vector second_start
	CADVector b = end - second; // vector second_end
	CADVector mid_a = (second + start)/2; // middle of vector a
	CADVector mid_b = (second + end)/2; // middle of vector b
	CADVector sym_a(-a.getY(), a.getX());
	CADVector sym_b(-b.getY(), b.getX());
	if (sym_a.getY() == 0.0)
		if (sym_a.getX() == 0.0) return;
		else if (sym_b.getY() == 0.0) return;
	if (sym_b.getX() == 0.0)
		if (sym_a.getX() == 0.0) return;
		else if (sym_b.getY() == 0.0) return;
	double lambda = 
		(sym_b.getY()*(mid_a.getX()-mid_b.getX()) - (sym_b.getX()*(mid_a.getY()-mid_b.getY())))
		/ (sym_a.getY()*sym_b.getX() - sym_a.getX()*sym_b.getY());
	this->center = mid_a + (sym_a*lambda);
	this->radius = (start - center).getLength();
	setStartAngle((start - center).getAngle());
	setEndAngle((end - center).getAngle());
}

/**
 * define the arc by startpoint, endpoint and included angle
 */
void CADArc::setStartEndPointAngle(CADVector start, CADVector end, double angle)
{
	CADVector mid = (end - start)/2;
	CADVector midStart = mid - start;
	CADVector sym(-midStart.getY(), midStart.getX());
	radius = midStart.getLength()/(sin(angle/2*(4*atan(1.0)/180.0)));
	double symlength = sqrt(pow(radius, 2)-pow(midStart.getLength(), 2));
	sym.setPolar(symlength, sym.getAngle());
	center = mid + sym;
	startAngle = (start - center).getAngle();
	endAngle = (end - center).getAngle();
}

/**
 * define the arc by startpoint, endpoint and radius
 */
void CADArc::setStartEndPointRadius(CADVector start, CADVector end, double radius)
{
	this->radius = radius;
	CADVector mid = (end - start)/2;
	CADVector midStart = mid - start;
	CADVector sym(-midStart.getY(), midStart.getX());
	double symlength = sqrt(pow(radius, 2)-pow(midStart.getLength(), 2));
	sym.setPolar(symlength, sym.getAngle());
	center = mid + sym;
	startAngle = (start - center).getAngle();
	endAngle = (end - center).getAngle();	
}

/** @return the Object "as String" */
QString CADArc::toString ()
{
	return QString::number(getEntityID())+":ARC:"+QString::number(center.getX())+","+
		QString::number(center.getY())+";"+QString::number(radius)+";"+
		QString::number(startAngle)+";"+QString::number(endAngle);
}

// EOF
