/***************************************************************************
 *                                                                         *
 *   src/cad/src/engine/cadvector.cpp                                      *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cad/engine/cadvector.h>

#include <qobject.h>
#include <cmath>

using cad::engine::CADVector;
using cad::util::Exception;

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

CADVector::CADVector(double x, double y, bool absolute)
{
	set(x, y);
	this->absolute = absolute;
}

CADVector::~CADVector()
{

}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

/** @return the x value */
double CADVector::getX() const
{
	return x;
}

/** @return the y value */
double CADVector::getY() const
{
	return y;
}

/** @return the vector length */
double CADVector::getLength() const
{
	return sqrt(pow(x, 2) + pow(y, 2));
}

/** @return the vector angle */
double CADVector::getAngle() const
{
	if (x == 0.0) return (y < 0) ? 270.0 : 90.0;
	if (y == 0.0) return (x < 0) ? 180.0 : 0.0;
	double angle = atan(y/x) * (180/(4.0*atan(1.0)));
	if (x < 0.0) angle += 180.0;
	return angle;
}

/** @return if the vector is absolute or relative */
bool CADVector::isAbsolute() const
{
	return absolute;
}

/** set the x and y values */
void CADVector::set(double x, double y)
{
	this->x = x;
	this->y = y;
}

/** set the polar values */
void CADVector::setPolar(double length, double angle)
{
	x = length * cos(angle / (180/(4*atan(1.0))));
	y = length * sin(angle / (180/(4*atan(1.0))));
}

/** set the x value */
void CADVector::setX(double x)
{
	this->x = x;
}

/** set the y value */
void CADVector::setY(double y)
{
	this->y = y;
}

/** set the vector absolute or relative */
void CADVector::setAbsolute(bool absolute)
{
	this->absolute = absolute;
}

CADVector CADVector::operator+(const CADVector& vector) const
{
	return CADVector(x+vector.x, y+vector.y, true);
}

CADVector CADVector::operator-(const CADVector& vector) const
{
	return CADVector(x-vector.x, y-vector.y, true);
}

CADVector CADVector::operator*(const double& d)
{
	return CADVector(x*d, y*d);
}

double CADVector::operator*(const CADVector& v)
{
	return x*v.getX() + y*v.getY();
}

CADVector CADVector::operator/(const double& d)
{
	return CADVector(x/d, y/d);
}

CADVector CADVector::mid(const CADVector& vector)
{
	return (*this+vector)/2;
}

CADVector CADVector::rotateLeft() const
{
	return CADVector(-y, x);
}

CADVector CADVector::rotateRight() const
{
	return CADVector(y, -x);
}

/** parse a vector from a string */
CADVector CADVector::parseString(QString s) throw (Exception)
{
	bool absolute = true;
	if (s[0] == '@')
	{
		absolute = false;
		s = s.mid(1);
	}
	
	CADVector vector;
	int position;
	if ((position = s.find(',')) != -1)
	{
		bool ok = false;
		QString number = s.left(position);
		double x = number.toDouble(&ok);
		if (!ok)
			throw Exception(QObject::tr("Invalid point."));
		number = s.mid(position + 1);
		double y = number.toDouble(&ok);
		if (!ok)
			throw Exception(QObject::tr("Invalid point."));
		vector.setX(x);
		vector.setY(y);
	}
	else if ((position = s.find('<')) != -1)
	{
		bool ok = false;
		QString number = s.left(position);
		double length = number.toDouble(&ok);
		if (!ok)
			throw Exception(QObject::tr("Invalid point."));
		number = s.mid(position + 1);
		double angle = number.toDouble(&ok);
		if (!ok)
			throw Exception(QObject::tr("Invalid point."));
		vector.setPolar(length, angle);
	}
	else
	{
		throw Exception(QObject::tr("Invalid point."));
	}
	
	vector.setAbsolute(absolute);
	return vector;
}

// EOF
