/***************************************************************************
 *                                                                         *
 *   src/concadgui/src/dialogs/colordialog.cpp                             *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include <concadgui/dialogs/colordialog.h>

#include <cad/engine/caddrawing.h>
#include <cad/engine/cadlayer.h>
#include <concadio/drawingmanager.h>

#include <concadgui/widgets/colorpalette.h>
#include <concadgui/widgets/graypalette.h>
#include <concadgui/widgets/completepalette.h>
#include <qlineedit.h>
#include <qmessagebox.h>

using namespace concadgui::widgets;
using concadgui::dialogs::ColorDialog;
using concadio::DrawingManager;
using cad::engine::CADColor;
using cad::util::Exception;

/* TRANSLATOR concadgui::dialogs::ColorDialog */

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/**
 * Constructor
 * Create a ColorDialog with the given color selected
 * @param layer: if layer is QString::null BYLAYER and BYBLOCK 
 * will be disabled
 */
ColorDialog::ColorDialog(CADColor color, QString layer, QWidget* parent, const char* name) : ColorDialogBase(parent, name)
{
	this->adjustSize();
	this->setFixedSize(width(), height());
	this->color = color;
	this->layer = layer;
	
	connect(standardColors, SIGNAL(selected(int)), this,
		SLOT(onColorChange(int)));
	connect(grayColors, SIGNAL(selected(int)), this,
		SLOT(onColorChange(int)));
	connect(completeColors, SIGNAL(selected(int)), this,
		SLOT(onColorChange(int)));
	
	updateColorDisplay();
}

/** Destructor */
ColorDialog::~ColorDialog()
{
	
}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

CADColor ColorDialog::getColor()
{
	return color;
}

/***************************************************************************
 * Public Methods                                                          *
 ***************************************************************************/

/** Converts a color name to the color value */
CADColor ColorDialog::nameToValue(QString name) throw (Exception)
{
	if (name == "BYBLOCK") return CADColor::BYBLOCK;
	if (name == "red") return CADColor::RED;
	if (name == "yellow") return CADColor::YELLOW;
	if (name == "green") return CADColor::GREEN;
	if (name == "cyan") return CADColor::CYAN;
	if (name == "blue") return CADColor::BLUE;
	if (name == "magenta") return CADColor::MAGENTA;
	if (name == "white") return CADColor::WHITE;
	if (name == "BYLAYER") return CADColor::BYLAYER;
	
	bool valid;
	int value = name.toInt(&valid);
	if ((!valid) || (value < 0) || (value > 256))
		throw Exception(tr("Invalid color name."));
	return CADColor(value);
}

/** Converts a color value to the color name */
QString ColorDialog::valueToName(CADColor color)
{
	switch (color.getValue())
	{
	case 0: return "BYBLOCK";
	case 1: return "red";
	case 2: return "yellow";
	case 3: return "green";
	case 4: return "cyan";
	case 5: return "blue";
	case 6: return "magenta";
	case 7: return "white";
	case 256: return "BYLAYER";
	default:
		if ((color.getValue() > 7) && (color.getValue() < 256))
			return QString::number(color.getValue());
	}
	return QString::null;
}

/***************************************************************************
 * Private Methods                                                         *
 ***************************************************************************/

/** update the color display (name, color) */
void ColorDialog::updateColorDisplay()
{
	colorName->setText(valueToName(color.getValue()));
	CADColor c = color;
	if (c == CADColor::BYLAYER)
	{
		try
		{
			c = DrawingManager::getInstance()->getDrawing()->getLayerList()
			.getLayer(layer).getPen().getColor();
		}
		catch (...)
		{
			c.setValue(7);
			QMessageBox::information(this, tr("Error"), tr("Couldn\'t get color from layer."));
		}
	}
	if (c == CADColor::BYBLOCK)
		c.setValue(7);
	colorDisplay->setBackgroundColor(QColor(c.getRed(), c.getGreen(), 
		c.getBlue()));
}

/***************************************************************************
 * Private Slots                                                           *
 ***************************************************************************/

void ColorDialog::onByLayerClick()
{
	color = CADColor::BYLAYER;
	updateColorDisplay();
}

void ColorDialog::onByBlockClick()
{
	color = CADColor::BYBLOCK;
	updateColorDisplay();
}

void ColorDialog::onColorChange(int color)
{
	this->color.setValue(color);
	updateColorDisplay();
}

void ColorDialog::onColorNameReturn()
{
	CADColor color;
	try
	{
		color = nameToValue(colorName->text());
		if ((layer == QString::null) && ((color == CADColor::BYLAYER) || (color == CADColor::BYBLOCK)))
		{
			QMessageBox::warning(this, tr("Error"), tr("Color can\'t be %1.").arg(colorName->text()));
			return;
		}
		this->color = color;
		updateColorDisplay();
	}
	catch (Exception e)
	{
		QMessageBox::warning(this, tr("Error"), e.getError());
	}
}

// EOF
