/***************************************************************************
 *                                                                         *
 *   src/concadgui/src/dialogs/layerdialog.cpp                             *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <concadgui/dialogs/layerdialog.h>

#include <concadgui/dialogs/colordialog.h>
#include <concadgui/dialogs/linetypedialog.h>
#include <concadio/drawingmanager.h>
#include <concadio/undoredo/undolayer.h>
#include <cad/engine/caddrawing.h>
#include <cad/util/exception.h>

#include <qlistview.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qmessagebox.h>

using namespace concadgui::dialogs;
using namespace cad::engine;
using namespace concadio::undoredo;
using concadio::DrawingManager;
using cad::util::Exception;

/* TRANSLATOR concadgui::dialogs::LayerDialog */

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/**
 * Constructor
 * Create a LayerDialog which contains the given layerlist
 */
LayerDialog::LayerDialog(CADLayerList layerList, QWidget* parent, char* name):
LayerDialogBase(parent, name)
{
	this->adjustSize();
	this->setFixedSize(width(), height());
	
	this->layerList = layerList;
	updateLayerView();
	layerView->setSelected(layerView->firstChild(), true);
	undo = new UndoAbleGroup();
}

/** Destructor */
LayerDialog::~LayerDialog()
{

}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

/** @return the layerlist */
CADLayerList LayerDialog::getLayerList()
{
	return layerList;
}

/***************************************************************************
 * Private Methods                                                         *
 ***************************************************************************/

/** update the layerlist display */
void LayerDialog::updateLayerView()
{
	layerView->clear();
	for (int i = 0; i<layerList.getCount(); i++)
	{
		CADLayer layer = layerList.getLayer(i);
		QListViewItem* item = new QListViewItem(layerView, layer.getName(),  QString(layer.getFrozen()?"f ":". ") + QString(layer.getLocked()?"l ":". "), ColorDialog::valueToName(layer.getPen().getColor().getValue()), layer.getPen().getLinetype());
		layerView->insertItem(item);
	}
	
	activeLayer->setText(tr("Active Layer: ") + layerList.getLayer(layerList.getActive()).getName());
}

/***************************************************************************
 * Private Slots                                                           *
 ***************************************************************************/

void LayerDialog::onNewClick()
{
	if (layerName->text().isEmpty())
	{
		QMessageBox::warning(this, tr("Error"), tr("Please insert a layer name."));
		return;
	}

	try
	{
		layerList.add(CADLayer(layerName->text()));
		updateLayerView();
		undo->addUndoAble(new UndoNewLayer(CADLayer(layerName->text())));
	}
	catch (Exception e)
	{
		QMessageBox::information(this, tr("Error"), e.getError());
	}
}

void LayerDialog::onDeleteClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			if (!DrawingManager::getInstance()->getDrawing()->objectsOnLayer(selected))
			{
				UndoDeleteLayer* undoLayer = new UndoDeleteLayer(layerList.getLayer(selected));
				layerList.remove(selected);
				updateLayerView();
				undo->addUndoAble(undoLayer);
			}
			else
				QMessageBox::information(this, tr("Error"), tr("Cannot delete a layer with objects on it."));
		}
		catch (Exception e)
		{
			QMessageBox::information(this, tr("Error"), e.getError());
		}
	}
	else
		QMessageBox::information(this, tr("Error"), tr("No layer has been selected."));
}

void LayerDialog::onSetActiveClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			UndoActiveLayer* undoLayer = new UndoActiveLayer(layerList.getLayer(layerList.getActive()).getName(), selected);
			layerList.setActive(selected);
			updateLayerView();
			undo->addUndoAble(undoLayer);
		}
		catch (Exception e)
		{
			QMessageBox::information(this, tr("Error"), e.getError());	
		}
	}
	else
		QMessageBox::information(this, tr("Error"), tr("No layer has been selected."));
}

void LayerDialog::onRenameClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			CADLayer layer = layerList.getLayer(selected);
			layer.setName(layerName->text());
			UndoChangeLayer* undoLayer = new UndoChangeLayer(layerList.getLayer(selected), layer);
			layerList.update(selected, layer);
			rm.add(selected, layer.getName());
			updateLayerView();
			undo->addUndoAble(undoLayer);
		}
		catch (Exception e)
		{
			QMessageBox::information(this, tr("Error"), e.getError());
		}
	}
	else
		QMessageBox::information(this, tr("Error"), tr("No layer has been selected."));
}

void LayerDialog::onFreezeClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			CADLayer layer = layerList.getLayer(selected);
			layer.freeze();
			UndoChangeLayer* undoLayer = new UndoChangeLayer(layerList.getLayer(selected), layer);
			layerList.update(layer.getName(), layer);
			updateLayerView();
			undo->addUndoAble(undoLayer);
		}
		catch (Exception e)
		{
			QMessageBox::warning(this, tr("Error"), e.getError());	
		}
	}
	else
		QMessageBox::warning(this, tr("Error"), tr("No layer has been selected."));
}

void LayerDialog::onThawClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			CADLayer layer = layerList.getLayer(selected);
			layer.thaw();
			UndoChangeLayer* undoLayer = new UndoChangeLayer(layerList.getLayer(selected), layer);
			layerList.update(layer.getName(), layer);
			updateLayerView();
			undo->addUndoAble(undoLayer);
		}
		catch (Exception e)
		{
			QMessageBox::warning(this, tr("Error"), e.getError());	
		}
	}
	else
		QMessageBox::warning(this, tr("Error"), tr("No layer has been selected."));
}

void LayerDialog::onLockClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			CADLayer layer = layerList.getLayer(selected);
			layer.lock();
			UndoChangeLayer* undoLayer = new UndoChangeLayer(layerList.getLayer(selected), layer);
			layerList.update(layer.getName(), layer);
			updateLayerView();
			undo->addUndoAble(undoLayer);
		}
		catch (Exception e)
		{
			QMessageBox::warning(this, tr("Error"), e.getError());	
		}
	}
	else
		QMessageBox::warning(this, tr("Error"), tr("No layer has been selected."));
}

void LayerDialog::onUnlockClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			CADLayer layer = layerList.getLayer(selected);
			layer.unlock();
			UndoChangeLayer* undoLayer = new UndoChangeLayer(layerList.getLayer(selected), layer);
			layerList.update(layer.getName(), layer);
			updateLayerView();
			undo->addUndoAble(undoLayer);
		}
		catch (Exception e)
		{
			QMessageBox::warning(this, tr("Error"), e.getError());	
		}
	}
	else
		QMessageBox::warning(this, tr("Error"), tr("No layer has been selected."));
}

void LayerDialog::onColorClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			CADLayer layer = layerList.getLayer(selected);
			ColorDialog dlg(layer.getPen().getColor(), QString::null, this);
			if (dlg.exec())
			{
				layer.setPen(CADPen(dlg.getColor(), layer.getPen().getLinetype()));
				UndoChangeLayer* undoLayer = new UndoChangeLayer(layerList.getLayer(selected), layer);
				layerList.update(layer.getName(), layer);
				updateLayerView();
				undo->addUndoAble(undoLayer);
			}
		}
		catch (Exception e)
		{
			QMessageBox::warning(this, tr("Error"), e.getError());	
		}
	}
	else
		QMessageBox::warning(this, tr("Error"), tr("No layer has been selected."));
}

void LayerDialog::onLinetypeClick()
{
	if (layerView->selectedItem() != 0)
	{
		try
		{
			QString selected = layerView->selectedItem()->text(0);
			CADLayer layer = layerList.getLayer(selected);
			LinetypeDialog dlg(DrawingManager::getInstance()->getDrawing()->getLinetypeList(), false, this);
			dlg.setLinetype(layer.getPen().getLinetype());
			if (dlg.exec())
			{
				layer.setPen(CADPen(layer.getPen().getColor(), dlg.getLinetype()));
				DrawingManager::getInstance()->getDrawing()->updateLinetypeList(dlg.getLinetypeList());
				undo->addUndoAble(dlg.getUndoActions());
				UndoChangeLayer* undoLayer = new UndoChangeLayer(layerList.getLayer(selected), layer);
				layerList.update(layer.getName(), layer);
				updateLayerView();
				undo->addUndoAble(undoLayer);
			}
		}
		catch (Exception e)
		{
			QMessageBox::warning(this, tr("Error"), e.getError());	
		}
	}
	else
		QMessageBox::warning(this, tr("Error"), tr("No layer has been selected."));
}

//EOF
