/***************************************************************************
 *                                                                         *
 *   src/concadgui/src/edit/propertiesedit.cpp                             *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <concadgui/edit/propertiesedit.h>
#include <concadgui/dialogs/colordialog.h>
#include <concadgui/dialogs/linetypedialog.h>
#include <concadgui/dialogs/layerselectiondialog.h>
#include <cad/engine/caddrawing.h>
#include <concadio/drawingmanager.h>

#include <qlineedit.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qvalidator.h>

using concadgui::edit::PropertiesEdit;
using namespace concadgui::dialogs;
using namespace cad::engine;
using concadio::DrawingManager;

/* TRANSLATOR concadgui::edit::PropertiesEdit */

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/**
 * Constructor
 * Create a PropertiesEdit which edits the given properties
 */
PropertiesEdit::PropertiesEdit(QWidget* parent, const char* name) : PropertiesEditBase(parent, name)
{
	this->adjustSize();
	this->setFixedSize(width(), height());
	linetypeFactorEdit->setValidator(new QDoubleValidator(linetypeFactorEdit));
}

/** Destructor */
PropertiesEdit::~PropertiesEdit()
{
	
}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

/** set the properties */
void PropertiesEdit::set(CADPen pen, QString layer)
{
	this->pen = pen;
	this->layer = layer;
	update();
}

/** @return the pen */
CADPen PropertiesEdit::getPen() throw (Exception)
{
	if (linetypeFactor->text() == "")
		throw Exception(tr("Please enter a value for the linetype factor."));
	double value = linetypeFactorEdit->text().toDouble();
	qDebug("linetype factor: "+QString::number(value));
	if (value <= 0.0)
		throw Exception(tr("Linetype factor must be greater than 0."));
	pen.setLinetypeFactor(value);
	return pen;
}

/** @return the layer */
QString PropertiesEdit::getLayer() const
{
	return layer;
}

/***************************************************************************
 * Private Functions                                                       *
 ***************************************************************************/

void PropertiesEdit::update()
{
	colorName->setText(ColorDialog::valueToName(pen.getColor().getValue()));
	CADColor c = pen.getColor();
	if (c == CADColor::BYLAYER)
	{
		try
		{
			c = DrawingManager::getInstance()->getDrawing()->getLayerList().
				getLayer(layer).getPen().getColor();
		}
		catch (...)
		{
			c.setValue(7);
			QMessageBox::information(this, tr("Error"), tr("Couldn\'t get color from layer."));
		}
	}
	else if (c == CADColor::BYBLOCK) c = CADColor::WHITE;
	colorDisplay->setBackgroundColor(QColor(c.getRed(), c.getGreen(),
		c.getBlue()));
	layerName->setText(layer);
	linetypeName->setText(pen.getLinetype());
	QString text;
	linetypeFactorEdit->setText(text.sprintf("%1.4f", pen.getLinetypeFactor()));
}

/***************************************************************************
 * Private Slots                                                           *
 ***************************************************************************/

void PropertiesEdit::onColorClick()
{
	ColorDialog dlg(pen.getColor(), layer, this);
	if (dlg.exec() == QDialog::Accepted)
	{
		pen.setColor(dlg.getColor());
		update();
	}
}

void PropertiesEdit::onLayerClick()
{
	LayerSelectionDialog dlg(DrawingManager::getInstance()->getDrawing()->getLayerList(), this);
	dlg.setLayer(layer);
	if (dlg.exec())
	{
		layer = dlg.getLayer();
		update();
	}
}

void PropertiesEdit::onLinetypeClick()
{
	LinetypeDialog dlg(DrawingManager::getInstance()->getDrawing()->getLinetypeList(), true, this);
	dlg.setLinetype(pen.getLinetype());
	if (dlg.exec())
	{
		pen.setLinetype(dlg.getLinetype());
		DrawingManager::getInstance()->getDrawing()->updateLinetypeList(dlg.getLinetypeList());
		update();
	}
}

void PropertiesEdit::onLinetypeFactorFocus()
{
	bool ok;
	double value = linetypeFactor->text().toDouble(&ok);
	if (ok)
	{
		pen.setLinetypeFactor(value);
		update();
	}
}

// EOF
