/***************************************************************************
 *                                                                         *
 *   src/concadgui/src/mainwindow.cpp                                      *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <concadgui/mainwindow.h>

#include <qapplication.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qdockarea.h>
#include <qevent.h>
#include <concadgui/widgets/commandwidget.h>
#include <concadgui/widgets/commandline.h>
#include <concadgui/widgets/entityview.h>
#include <concadgui/dialogs/aboutdialog.h>
#include <concadgui/dialogs/colordialog.h>
#include <concadgui/dialogs/layerdialog.h>
#include <concadgui/dialogs/linetypedialog.h>
#include <concadgui/dialogs/newdialog.h>

#include <concadio/drawingmanager.h>
#include <cad/engine/caddrawing.h>

using namespace concadgui;
using namespace concadgui::dialogs;
using namespace concadgui::widgets;
using namespace cad::engine;

/* TRANSLATOR concadgui::MainWindow */

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

MainWindow::MainWindow(QWidget* parent, char* name): QMainWindow(parent, name)
{
	this->setCaption("ConstructCAD");
	this->setIcon(QPixmap::fromMimeSource("concad-48x48.png"));
	this->resize(800,600);
	
	drawingManager = DrawingManager::getInstance();
	entityView = new EntityView(QDockWindow::InDock, this, 0, 0);
	addDockWindow (entityView,"EntityView", DockTornOff);
	entityView->setResizeEnabled(true);
	entityView->setFixedExtentWidth(110);
	
	createActions();
	createStandardToolBar();
	createPropertiesToolBar();
	createDrawToolBar();
	createModifyToolBar();
	createOSnapToolBar();
	createDimToolBar();
	createZoomToolBar();
	createMenu();
	initializeStatusBar();

	drawing = new QWidget(this);
	drawing->setBackgroundColor(Qt::black);
	this->setCentralWidget(drawing);
	
	commandWidget = new CommandWidget(this, QDockWindow::InDock, this, 0, 0);
	addDockWindow (commandWidget, tr("Command Line"), DockBottom);
	commandWidget->getCommandLine()->setFocus();
	printLine("ConstructCAD Version 0.2");
	printLine("Have a lot of fun...");
	setIOState(IO_Ready);
	connect(commandWidget, SIGNAL(input(QString& )), this, SLOT(commandInput(QString& )));
	connect(commandWidget, SIGNAL(escapePressed()), this, SLOT(escape()));
	
	updatePropertiesDisplay();
}

MainWindow::~MainWindow()
{
	delete DrawingManager::getInstance();
}

/***************************************************************************
 * Public Methods                                                          *
 ***************************************************************************/

QString MainWindow::readInput() throw (Exception)
{
	setIOState(IO_Input);
	while (getIOState() != IO_Finished)
	{
		if (getIOState() == IO_Cancelled) throw Exception();
		qApp->processEvents();
	}
	setIOState(IO_Finished);
	return lastInput;
}

void MainWindow::print(QString s)
{
	commandWidget->print(s);
}

void MainWindow::printLine(QString s)
{
	commandWidget->printLine(s);
}

void MainWindow::draw(CADDrawing* drawing)
{
	entityView->refresh(drawing);
}

void MainWindow::updatePropertiesDisplay()
{
	CADDrawing* drawing = drawingManager->getDrawing();
	if ((layerSelection != 0) && (linetypeSelection != 0))
	{
		layerSelection->clear();
		linetypeSelection->clear();
		
		CADLayerList layerList = drawing->getLayerList();
		int i;
		for (i = 0; i<layerList.getCount(); i++)
		{
			CADLayer layer = layerList.getLayer(i);
			
			QPixmap color(10, 10);
			QPainter p(&color);
			p.setPen(Qt::black);
			p.setBrush(QColor(layer.getPen().getColor().getRed(), layer.getPen().getColor().getGreen(), layer.getPen().getColor().getBlue()));
			p.drawRect(0, 0, 10, 10);
			
			layerSelection->insertItem(color, layer.getName());
			
		}
		QString currentLayer = layerList.getLayer(layerList.getActive()).getName();
		for (i = 0; i<layerSelection->count(); i++)
		{
			if (layerSelection->text(i) == currentLayer)
			{
				layerSelection->setCurrentItem(i);
				break;
			}
		}
		
		CADColor color = drawing->getActivePen().getColor();
		if (color == CADColor::BYLAYER)
		{
			// get color from layer
			color = drawing->getLayerList().getLayer(
			drawing->getLayerList().getActive()).getPen().getColor();
			// load "bylayer" icon
			QPixmap colorDisplay(QPixmap::fromMimeSource("bylayer.png"));
			QPainter p(&colorDisplay);
			// draw color from layer into the icon
			QPointArray points(3);
			points.setPoint(0, 3, 1);
			points.setPoint(1, 14, 1);
			points.setPoint(2, 14, 12);
			p.setPen(QColor(color.getRed(), color.getGreen(), color.getBlue()));
			p.setBrush(QColor(color.getRed(), color.getGreen(), color.getBlue()));
			p.drawPolygon(points);
			propertiesColorAction->setIconSet(QIconSet(colorDisplay));
		}
		else if (color == CADColor::BYBLOCK)
		{
			propertiesColorAction->setIconSet(QIconSet(QPixmap::fromMimeSource("byblock.png")));
		}
		else
		{
			QPixmap colorDisplay(QPixmap::fromMimeSource("ddcolor.png"));
			QPainter p(&colorDisplay);
			p.setPen(QColor(color.getRed(), color.getGreen(), color.getBlue()));
			p.setBrush(QColor(color.getRed(), color.getGreen(), color.getBlue()));
			p.drawRect(1, 1, 14, 14);
			propertiesColorAction->setIconSet(QIconSet(colorDisplay));
		}
		
		CADLinetypeList linetypeList = drawing->getLinetypeList();
		for (i = 0; i<linetypeList.getCount(); i++)
		{
			linetypeSelection->insertItem(linetypeList.getLinetype(i).getName());
		}
		QString linetype = drawing->getActivePen().getLinetype();
		for (i = 0; i<linetypeSelection->count(); i++)
		{
			if (linetypeSelection->text(i) == linetype)
			{
				linetypeSelection->setCurrentItem(i);
				break;
			}
		}
	}
}

void MainWindow::quitApplication()
{
	if (!isLocked())
	{
		qApp->quit();
	}
}

void MainWindow::lock()
{
	IOInterface::lock();
	propertiesLayerAction->setEnabled(false);
	layerSelection->setEnabled(false);
	propertiesColorAction->setEnabled(false);
	propertiesLinetypeAction->setEnabled(false);
	linetypeSelection->setEnabled(false);
	propertiesObjectAction->setEnabled(false);
	propertiesPropertiesAction->setEnabled(false);
}

void MainWindow::unlock()
{
	IOInterface::unlock();
	propertiesLayerAction->setEnabled(true);
	layerSelection->setEnabled(true);
	propertiesColorAction->setEnabled(true);
	propertiesLinetypeAction->setEnabled(true);
	linetypeSelection->setEnabled(true);
	propertiesObjectAction->setEnabled(true);
	propertiesPropertiesAction->setEnabled(true);
	
}

bool MainWindow::getNewDrawing(CADDrawing& drawing)
{
	NewDialog dlg(this);
	if (dlg.exec())
	{
		drawing.setDrawingName(dlg.getDrawingName());
		drawing.setFileName(dlg.getFileName());
		return true;
	}
	return false;
}

bool MainWindow::getLayerChanges(CADLayerList &layerList, LayerRenamingManager &rm,
	UndoAble *&undoAble)
{
	LayerDialog dlg(layerList, this);
	if (dlg.exec())
	{
		layerList = dlg.getLayerList();
		rm = dlg.getLayerRenamingManager();
		undoAble = dlg.getUndoActions();
		return true;
	}
	return false;
}

bool MainWindow::getColor(CADColor& color, QString activeLayer)
{
	ColorDialog dlg(color, activeLayer, this);
	if (dlg.exec())
	{
		color = dlg.getColor();
		return true;
	}
	return false;
}

bool MainWindow::getLinetype(QString& linetype, CADLinetypeList& list, 
	bool allowByLayer, UndoAbleGroup *&undoAble)
{
	LinetypeDialog dlg(list, allowByLayer, this);
	dlg.setLinetype(linetype);
	if (dlg.exec())
	{
		linetype = dlg.getLinetype();
		list = dlg.getLinetypeList();
		undoAble = dlg.getUndoActions();
		return true;
	}
	return false;
}

bool MainWindow::getActiveProperties()
{
	PropertiesDialog dlg(0, this);
	return dlg.exec();
}

CADEntity* MainWindow::getProperties(CADEntity *entity)
{
	PropertiesDialog dlg(entity, this);
	if (dlg.exec())
		return dlg.getEntity();
	return 0;
}

/***************************************************************************
 * Protected Methods                                                       *
 ***************************************************************************/

void MainWindow::resizeEvent(QResizeEvent* e)
{

}

/***************************************************************************
 * Private Methods                                                         *
 ***************************************************************************/

void MainWindow::createMenu()
{
	createFileMenu();
	createEditMenu();
	createToolsMenu();
	createHelpMenu();
}

void MainWindow::createFileMenu()
{
	fileMenu = new QPopupMenu(this);
	fileNewAction->addTo(fileMenu);
	fileOpenAction->addTo(fileMenu);
	fileMenu->insertSeparator();
	fileSaveAction->addTo(fileMenu);
	fileSaveAsAction->addTo(fileMenu);
	fileMenu->insertSeparator();
	filePlotAction->addTo(fileMenu);
	fileMenu->insertSeparator();
	fileQuitAction->addTo(fileMenu);
	menuBar()->insertItem(tr("&File"), fileMenu);
}

void MainWindow::createEditMenu()
{
	editMenu = new QPopupMenu(this);
	editUndoAction->addTo(editMenu);
	editRedoAction->addTo(editMenu);
	editMenu->insertSeparator();
	editCutAction->addTo(editMenu);
	editCopyAction->addTo(editMenu);
	editPasteAction->addTo(editMenu);
	menuBar()->insertItem(tr("&Edit"), editMenu);
}

void MainWindow::createToolsMenu()
{
	toolsMenu = new QPopupMenu(this);
	createToolsDrawMenu();
	createToolsModifyMenu();
	createToolsOSnapMenu();
	createToolsDimMenu();
	createToolsZoomMenu();

	toolsMenu->insertSeparator();
	toolsToolboxesMenu = new QPopupMenu(this);
	QPopupMenu* temp = createDockWindowMenu();
	toolsMenu->insertItem(tr("&Toolboxes"), temp);
	toolsOptionsAction->addTo(toolsMenu);
	toolsEntityViewAction->addTo(toolsMenu);
	menuBar()->insertItem(tr("&Tools"), toolsMenu);
}

void MainWindow::createToolsDrawMenu()
{
	toolsDrawMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("&Draw"), toolsDrawMenu);
	drawLineAction->addTo(toolsDrawMenu);
	drawXLineAction->addTo(toolsDrawMenu);
	drawRayAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawPLineAction->addTo(toolsDrawMenu);
	drawMLineAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawArcAction->addTo(toolsDrawMenu);
	drawArcContinueAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawCircleAction->addTo(toolsDrawMenu);
	drawRingAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawEllipseAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawPolygonAction->addTo(toolsDrawMenu);
	drawRectangleAction->addTo(toolsDrawMenu);
	drawSolidAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawPointAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawBlockAction->addTo(toolsDrawMenu);
	drawBlockInsertAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawHatchAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawTextAction->addTo(toolsDrawMenu);
	drawTextSLAction->addTo(toolsDrawMenu);
}

void MainWindow::createToolsModifyMenu()
{
	toolsModifyMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("&Modify"), toolsModifyMenu);
	modifyMoveAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyCopyAction->addTo(toolsModifyMenu);
	modifyOffsetAction->addTo(toolsModifyMenu);
	modifyMirrorAction->addTo(toolsModifyMenu);
	modifyArrayRectAction->addTo(toolsModifyMenu);
	modifyArrayPolarAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyRotateAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyStretchAction->addTo(toolsModifyMenu);
	modifyScaleAction->addTo(toolsModifyMenu);
	modifyLengthenAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyTrimAction->addTo(toolsModifyMenu);
	modifyExtendAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyBreakAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyEditTextAction->addTo(toolsModifyMenu);
	modifyHatchEditAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyChamferAction->addTo(toolsModifyMenu);
	modifyFilletAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyExplodeAction->addTo(toolsModifyMenu);
	modifyEraseAction->addTo(toolsModifyMenu);
}

void MainWindow::createToolsOSnapMenu()
{
	toolsSnapMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("Object &Snap"), toolsSnapMenu);
	snapFromPoint->addTo(toolsSnapMenu);
	snapEndpoint->addTo(toolsSnapMenu);
	snapMidpoint->addTo(toolsSnapMenu);
	snapIntersection->addTo(toolsSnapMenu);
	snapCenter->addTo(toolsSnapMenu);
	snapQuadrant->addTo(toolsSnapMenu);
	snapPerpendicular->addTo(toolsSnapMenu);
	snapTangent->addTo(toolsSnapMenu);
	snapNode->addTo(toolsSnapMenu);
	snapBasepoint->addTo(toolsSnapMenu);
	snapNearest->addTo(toolsSnapMenu);
	snapNon->addTo(toolsSnapMenu);
	snapObject->addTo(toolsSnapMenu);
	snapCalculator->addTo(toolsSnapMenu);
}

void MainWindow::createToolsDimMenu()
{
	toolsDimensioningMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("D&imensioning"), toolsDimensioningMenu);
	dimLinear->addTo(toolsDimensioningMenu);
	dimAligned->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimRadius->addTo(toolsDimensioningMenu);
	dimDiameter->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimAngular->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimOrdinate->addTo(toolsDimensioningMenu);
	dimOrdinateX->addTo(toolsDimensioningMenu);
	dimOrdinateY->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimBaseline->addTo(toolsDimensioningMenu);
	dimContinue->addTo(toolsDimensioningMenu);
	dimCenter->addTo(toolsDimensioningMenu);
	dimLeader->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimTEdit->addTo(toolsDimensioningMenu);
	dimTERotate->addTo(toolsDimensioningMenu);
	dimTELeft->addTo(toolsDimensioningMenu);
	dimTEHome->addTo(toolsDimensioningMenu);
	dimTERight->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimStyle->addTo(toolsDimensioningMenu);
	dimEdit->addTo(toolsDimensioningMenu);
}

void MainWindow::createToolsZoomMenu()
{
	toolsZoomMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("&Zoom"), toolsZoomMenu);

	zoomInAction->addTo(toolsZoomMenu);
	zoomOutAction->addTo(toolsZoomMenu);
	zoomWindowAction->addTo(toolsZoomMenu);
	zoomAllAction->addTo(toolsZoomMenu);
	zoomPreviousAction->addTo(toolsZoomMenu);
	zoomScaleAction->addTo(toolsZoomMenu);
	zoomExtentAction->addTo(toolsZoomMenu);
	zoomCenterAction->addTo(toolsZoomMenu);
}

void MainWindow::createHelpMenu()
{
	helpMenu = new QPopupMenu(this);
	helpHelpAction->addTo(helpMenu);
	helpMenu->insertSeparator();
	helpAboutAction->addTo(helpMenu);
	menuBar()->insertItem(tr("&Help"), helpMenu);
}

void MainWindow::createStandardToolBar()
{
	standardToolBar = new QToolBar(tr("Standard"), this, this, TRUE);
	addDockWindow(standardToolBar, tr("Standard"), DockTop, TRUE);

	fileNewAction->addTo(standardToolBar);
	fileOpenAction->addTo(standardToolBar);
	fileSaveAction->addTo(standardToolBar);
	filePlotAction->addTo(standardToolBar);
	standardToolBar->addSeparator();
	editCutAction->addTo(standardToolBar);
	editCopyAction->addTo(standardToolBar);
	editPasteAction->addTo(standardToolBar);
	standardToolBar->addSeparator();
	editUndoAction->addTo(standardToolBar);
	editRedoAction->addTo(standardToolBar);
	standardToolBar->addSeparator();
	zoomInAction->addTo(standardToolBar);
	zoomOutAction->addTo(standardToolBar);
	zoomWindowAction->addTo(standardToolBar);
	zoomAllAction->addTo(standardToolBar);
	standardToolBar->addSeparator();
	helpHelpAction->addTo(standardToolBar);
}

void MainWindow::createPropertiesToolBar()
{
	propertiesToolBar = new QToolBar(tr("Properties"), this, this, TRUE);
	addDockWindow(propertiesToolBar, tr("Properties"), DockTop, TRUE);

	propertiesLayerAction->addTo(propertiesToolBar);

	layerSelection = new QComboBox(propertiesToolBar);
	layerSelection->setFixedSize(150,24);
	connect(layerSelection, SIGNAL(activated(const QString&)), this, SLOT(onPropertiesLayerChange(const QString&)));

	propertiesColorAction->addTo(propertiesToolBar);
	propertiesLinetypeAction->addTo(propertiesToolBar);

	linetypeSelection = new QComboBox(propertiesToolBar);
	linetypeSelection->setFixedSize(150,24);
	connect(linetypeSelection, SIGNAL(activated(const QString&)), this, SLOT(onPropertiesLinetypeChange(const QString&)));

	propertiesObjectAction->addTo(propertiesToolBar);
	propertiesPropertiesAction->addTo(propertiesToolBar);
}

void MainWindow::createDrawToolBar()
{
	drawToolBar = new QToolBar(tr("Draw"), this, this, TRUE);
	addDockWindow(drawToolBar, tr("Draw"), DockLeft, TRUE);

	drawLineAction->addTo(drawToolBar);
	drawXLineAction->addTo(drawToolBar);
	drawPLineAction->addTo(drawToolBar);
	drawMLineAction->addTo(drawToolBar);
	drawArcAction->addTo(drawToolBar);
	drawCircleAction->addTo(drawToolBar);
	drawEllipseAction->addTo(drawToolBar);
	drawPolygonAction->addTo(drawToolBar);
	drawRectangleAction->addTo(drawToolBar);
	drawPointAction->addTo(drawToolBar);
	drawBlockAction->addTo(drawToolBar);
	drawBlockInsertAction->addTo(drawToolBar);
	drawHatchAction->addTo(drawToolBar);
	drawTextAction->addTo(drawToolBar);
}

void MainWindow::createModifyToolBar()
{
	modifyToolBar = new QToolBar(tr("Modify"), this, this, TRUE);
	addDockWindow(modifyToolBar, tr("Modify"), DockLeft, TRUE);

	modifyMoveAction->addTo(modifyToolBar);
	modifyCopyAction->addTo(modifyToolBar);
	modifyOffsetAction->addTo(modifyToolBar);
	modifyMirrorAction->addTo(modifyToolBar);
	modifyArrayRectAction->addTo(modifyToolBar);
	modifyRotateAction->addTo(modifyToolBar);
	modifyStretchAction->addTo(modifyToolBar);
	modifyTrimAction->addTo(modifyToolBar);
	modifyExtendAction->addTo(modifyToolBar);
	modifyBreakAction->addTo(modifyToolBar);
	modifyChamferAction->addTo(modifyToolBar);
	modifyFilletAction->addTo(modifyToolBar);
	modifyExplodeAction->addTo(modifyToolBar);
	modifyEraseAction->addTo(modifyToolBar);
}

void MainWindow::createOSnapToolBar()
{
	osnapToolBar = new QToolBar(tr("Object Snap"), this, this, TRUE);
	addDockWindow(osnapToolBar, tr("Object Snap"), DockTornOff, TRUE);

	snapFromPoint->addTo(osnapToolBar);
	snapEndpoint->addTo(osnapToolBar);
	snapMidpoint->addTo(osnapToolBar);
	snapIntersection->addTo(osnapToolBar);
	snapCenter->addTo(osnapToolBar);
	snapQuadrant->addTo(osnapToolBar);
	snapPerpendicular->addTo(osnapToolBar);
	snapTangent->addTo(osnapToolBar);
	snapNode->addTo(osnapToolBar);
	snapBasepoint->addTo(osnapToolBar);
	snapNearest->addTo(osnapToolBar);
	snapNon->addTo(osnapToolBar);
	snapObject->addTo(osnapToolBar);
	snapCalculator->addTo(osnapToolBar);
}

void MainWindow::createDimToolBar()
{
	dimToolBar = new QToolBar(tr("Dimensioning"), this, this, TRUE);
	addDockWindow(dimToolBar, tr("Dimensioning"), DockTornOff, TRUE);
	
	dimLinear->addTo(dimToolBar);
	dimAligned->addTo(dimToolBar);
	dimRadius->addTo(dimToolBar);
	dimDiameter->addTo(dimToolBar);
	dimAngular->addTo(dimToolBar);
	dimOrdinate->addTo(dimToolBar);
	dimBaseline->addTo(dimToolBar);
	dimContinue->addTo(dimToolBar);
	dimCenter->addTo(dimToolBar);
	dimLeader->addTo(dimToolBar);
	dimTEdit->addTo(dimToolBar);
	dimStyle->addTo(dimToolBar);
}

void MainWindow::createZoomToolBar()
{
	zoomToolBar = new QToolBar(tr("Zoom"), this, this, TRUE);
	addDockWindow(zoomToolBar, tr("Zoom"), DockTornOff, TRUE);
	
	zoomInAction->addTo(zoomToolBar);
	zoomOutAction->addTo(zoomToolBar);
	zoomWindowAction->addTo(zoomToolBar);
	zoomAllAction->addTo(zoomToolBar);
	zoomPreviousAction->addTo(zoomToolBar);
	zoomScaleAction->addTo(zoomToolBar);
	zoomExtentAction->addTo(zoomToolBar);
	zoomCenterAction->addTo(zoomToolBar);
}

void MainWindow::initializeStatusBar()
{
	status[0] = new QLabel(statusBar());
	status[0]->setText(tr("Ready"));
	status[0]->setFixedWidth(100);
	statusBar()->addWidget(status[0]);
}

void MainWindow::createActions()
{
	createPropertiesActions();
	createDrawActions();
	createModifyActions();
	createSnapActions();
	createDimActions();
	createZoomActions();

	createFileActions();
	createEditActions();
	createToolsActions();
	createHelpActions();
}

void MainWindow::createFileActions()
{
	fileNewAction = new QAction(QIconSet(QPixmap::fromMimeSource("new.png")), tr("&New"), CTRL+Key_N, this);
	connect(fileNewAction, SIGNAL(activated()), this, SLOT(onFileNew()));
	fileOpenAction = new QAction(QIconSet(QPixmap::fromMimeSource("open.png")), tr("&Open"), CTRL+Key_O, this);
	connect(fileOpenAction, SIGNAL(activated()), this, SLOT(onFileOpen()));
	fileSaveAction = new QAction(QIconSet(QPixmap::fromMimeSource("save.png")), tr("&Save"), CTRL+Key_S, this);
	connect(fileSaveAction, SIGNAL(activated()), this, SLOT(onFileSave()));
	fileSaveAsAction = new QAction(tr("Save &As..."), 0, this);
	connect(fileSaveAsAction, SIGNAL(activated()), this, SLOT(onFileSaveAs()));
	filePlotAction = new QAction(QIconSet(QPixmap::fromMimeSource("plot.png")), tr("&Plot"), CTRL+Key_P, this);
	connect(filePlotAction, SIGNAL(activated()), this, SLOT(onFilePlot()));
	fileQuitAction = new QAction(tr("&Quit"), CTRL+Key_Q, this);
	connect(fileQuitAction, SIGNAL(activated()), this, SLOT(onFileQuit()));
}

void MainWindow::createEditActions()
{
	editUndoAction = new QAction(QIconSet(QPixmap::fromMimeSource("undo.png")), tr("&Undo"), CTRL+Key_Z, this);
	connect(editUndoAction, SIGNAL(activated()), this, SLOT(onEditUndo()));
	editRedoAction = new QAction(QIconSet(QPixmap::fromMimeSource("redo.png")), tr("&Redo"), CTRL+Key_Y, this);
	connect(editRedoAction, SIGNAL(activated()), this, SLOT(onEditRedo()));
	editCutAction = new QAction(QIconSet(QPixmap::fromMimeSource("cut.png")), tr("&Cut"), CTRL+Key_X, this);
	connect(editCutAction, SIGNAL(activated()), this, SLOT(onEditCut()));
	editCopyAction = new QAction(QIconSet(QPixmap::fromMimeSource("copy.png")), tr("&Copy"), CTRL+Key_C, this);
	connect(editCopyAction, SIGNAL(activated()), this, SLOT(onEditCopy()));
	editPasteAction = new QAction(QIconSet(QPixmap::fromMimeSource("paste.png")), tr("&Paste"), CTRL+Key_V, this);
	connect(editPasteAction, SIGNAL(activated()), this, SLOT(onEditPaste()));
}

void MainWindow::createToolsActions()
{
	toolsOptionsAction = new QAction(tr("&Options"), 0, this);
	connect(toolsOptionsAction, SIGNAL(activated()), this, SLOT(onToolsOptions()));
	toolsEntityViewAction = new QAction("&Entity View", 0, this);
	connect(toolsEntityViewAction, SIGNAL(activated()), this, SLOT(onToolsEntityView()));
}

void MainWindow::createHelpActions()
{
	helpHelpAction = new QAction(QIconSet(QPixmap::fromMimeSource("help.png")), tr("&Help"), Key_F1, this);
	connect(helpHelpAction, SIGNAL(activated()), this, SLOT(onHelpHelp()));
	helpAboutAction = new QAction(tr("&About ConstructCAD"), 0, this);
	connect(helpAboutAction, SIGNAL(activated()), this, SLOT(onHelpAbout()));
}

void MainWindow::createPropertiesActions()
{
	propertiesLayerAction = new QAction(QIconSet(QPixmap::fromMimeSource("layer.png")), tr("&Layers"), 0, this);
	connect(propertiesLayerAction, SIGNAL(activated()), this, SLOT(onPropertiesLayer()));
	propertiesColorAction = new QAction(tr("C&olor"), 0, this);
	connect(propertiesColorAction, SIGNAL(activated()), this, SLOT(onPropertiesColor()));
	propertiesLinetypeAction = new QAction(QIconSet(QPixmap::fromMimeSource("ddltype.png")), tr("Linet&ype"), 0, this);
	connect(propertiesLinetypeAction, SIGNAL(activated()), this, SLOT(onPropertiesLinetype()));
	propertiesObjectAction = new QAction(QIconSet(QPixmap::fromMimeSource("ddemodes.png")), tr("Ob&ject Properties"), 0, this);
	connect(propertiesObjectAction, SIGNAL(activated()), this, SLOT(onPropertiesObject()));
	propertiesPropertiesAction = new QAction(QIconSet(QPixmap::fromMimeSource("properties.png")), tr("&Properties"), 0, this);
	connect(propertiesPropertiesAction, SIGNAL(activated()), this,
    	SLOT(onPropertiesProperties()));
}

void MainWindow::createZoomActions()
{
	zoomInAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomin.png")), tr("Zoom &In"), 0, this);
	connect(zoomInAction, SIGNAL(activated()), this, SLOT(onZoomIn()));
	zoomOutAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomout.png")), tr("Zoom &Out"), 0, this);
	connect(zoomOutAction, SIGNAL(activated()), this, SLOT(onZoomOut()));
	zoomWindowAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomwindow.png")), tr("Zoom &Window"), 0, this);
	connect(zoomWindowAction, SIGNAL(activated()), this, SLOT(onZoomWindow()));
	zoomAllAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomall.png")), tr("Zoom &All"), 0, this);
	connect(zoomAllAction, SIGNAL(activated()), this, SLOT(onZoomAll()));
	zoomPreviousAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomprev.png")), tr("Zoom &Previous"), 0, this);
	connect(zoomPreviousAction, SIGNAL(activated()), this, SLOT(onZoomPrevious()));
	zoomScaleAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomscale.png")), tr("Zoom &Scale"), 0, this);
	connect(zoomScaleAction, SIGNAL(activated()), this, SLOT(onZoomScale()));
	zoomExtentAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomextent.png")), tr("Zoom &Extent"), 0, this);
	connect(zoomExtentAction, SIGNAL(activated()), this, SLOT(onZoomExtent()));
	zoomCenterAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomcenter.png")), tr("Zoom &Center"), 0, this);
	connect(zoomCenterAction, SIGNAL(activated()), this, SLOT(onZoomCenter()));
}

void MainWindow::createDrawActions()
{
	drawLineAction = new QAction(QIconSet(QPixmap::fromMimeSource("line.png")), tr("Line"), 0, this);
	connect(drawLineAction, SIGNAL(activated()), this, SLOT(onDrawLine()));
	drawXLineAction = new QAction(QIconSet(QPixmap::fromMimeSource("xline.png")), tr("Construction Line"), 0, this);
	connect(drawXLineAction, SIGNAL(activated()), this, SLOT(onDrawXLine()));
	drawRayAction = new QAction(QIconSet(QPixmap::fromMimeSource("ray.png")), tr("Ray"), 0, this);
	connect(drawRayAction, SIGNAL(activated()), this, SLOT(onDrawRay()));
	drawPLineAction = new QAction(QIconSet(QPixmap::fromMimeSource("pline.png")), tr("Polyline"), 0, this);
	connect(drawPLineAction, SIGNAL(activated()), this, SLOT(onDrawPLine()));
	drawMLineAction = new QAction(QIconSet(QPixmap::fromMimeSource("mline.png")), tr("Multiline"), 0, this);
	connect(drawMLineAction, SIGNAL(activated()), this, SLOT(onDrawMLine()));
	drawArcAction = new QAction(QIconSet(QPixmap::fromMimeSource("arc.png")), tr("Arc"), 0, this);
	connect(drawArcAction, SIGNAL(activated()), this, SLOT(onDrawArc()));
	drawArcContinueAction = new QAction(QIconSet(QPixmap::fromMimeSource("arccont.png")), tr("Continue Arc"), 0, this);
	connect(drawArcContinueAction, SIGNAL(activated()), this, SLOT(onDrawArcContinue()));
	drawCircleAction = new QAction(QIconSet(QPixmap::fromMimeSource("circle.png")), tr("Circle"), 0, this);
	connect(drawCircleAction, SIGNAL(activated()), this, SLOT(onDrawCircle()));
	drawRingAction = new QAction(QIconSet(QPixmap::fromMimeSource("ring.png")), tr("Ring"), 0, this);
	connect(drawRingAction, SIGNAL(activated()), this, SLOT(onDrawRing()));
	drawEllipseAction = new QAction(QIconSet(QPixmap::fromMimeSource("ellipse.png")), tr("Ellipse"), 0, this);
	connect(drawEllipseAction, SIGNAL(activated()), this, 
    	SLOT(onDrawEllipse()));
	drawPolygonAction = new QAction(QIconSet(QPixmap::fromMimeSource("polygon.png")), tr("Polygon"), 0, this);
	connect(drawPolygonAction, SIGNAL(activated()), this, SLOT(onDrawPolygon()));
	drawRectangleAction = new QAction(QIconSet(QPixmap::fromMimeSource("rectangle.png")), tr("Rectangle"), 0, this);
	connect(drawRectangleAction, SIGNAL(activated()), this, SLOT(onDrawRectangle()));
	drawSolidAction = new QAction(QIconSet(QPixmap::fromMimeSource("solid.png")), tr("Solid"), 0, this);
	connect(drawSolidAction, SIGNAL(activated()), this, SLOT(onDrawSolid()));
	drawPointAction = new QAction(QIconSet(QPixmap::fromMimeSource("point.png")), tr("Point"), 0, this);
	connect(drawPointAction, SIGNAL(activated()), this, SLOT(onDrawPoint()));
	drawBlockAction = new QAction(QIconSet(QPixmap::fromMimeSource("block.png")), tr("Block"), 0, this);
	connect(drawBlockAction, SIGNAL(activated()), this, SLOT(onDrawBlock()));
	drawBlockInsertAction = new QAction(QIconSet(QPixmap::fromMimeSource("blockins.png")), tr("Insert Block"), 0, this);
	connect(drawBlockInsertAction, SIGNAL(activated()), this, SLOT(onDrawBlockInsert()));
	drawHatchAction = new QAction(QIconSet(QPixmap::fromMimeSource("hatch.png")), tr("Hatching"), 0, this);
	connect(drawHatchAction, SIGNAL(activated()), this, SLOT(onDrawHatch()));
	drawTextAction = new QAction(QIconSet(QPixmap::fromMimeSource("text.png")), tr("Text"), 0, this);
	connect(drawTextAction, SIGNAL(activated()), this, SLOT(onDrawText()));
	drawTextSLAction = new QAction(QIconSet(QPixmap::fromMimeSource("textsl.png")), tr("Single-Line Text"), 0, this);
	connect(drawTextSLAction, SIGNAL(activated()), this, SLOT(onDrawTextSL()));
}

void MainWindow::createModifyActions()
{
	modifyMoveAction = new QAction(QIconSet(QPixmap::fromMimeSource("move.png")), tr("Move"), 0, this);
	connect(modifyMoveAction, SIGNAL(activated()), this, SLOT(onModifyMove()));
    modifyCopyAction = new QAction(QIconSet(QPixmap::fromMimeSource("objectcopy.png")), tr("Copy"), 0, this);
	connect(modifyCopyAction, SIGNAL(activated()), this, SLOT(onModifyCopy()));
	modifyOffsetAction = new QAction(QIconSet(QPixmap::fromMimeSource("offset.png")), tr("Offset"), 0, this);
	connect(modifyOffsetAction, SIGNAL(activated()), this, SLOT(onModifyOffset()));
	modifyMirrorAction = new QAction(QIconSet(QPixmap::fromMimeSource("mirror.png")), tr("Mirror"), 0, this);
	connect(modifyMirrorAction, SIGNAL(activated()), this, SLOT(onModifyMirror()));
	modifyArrayRectAction = new QAction(QIconSet(QPixmap::fromMimeSource("arrayrect.png")), tr("Rectangular Array"), 0, this);
	connect(modifyArrayRectAction, SIGNAL(activated()), this, SLOT(onModifyArrayRect()));
	modifyArrayPolarAction = new QAction(QIconSet(QPixmap::fromMimeSource("arraypolar.png")), tr("Polar Array"), 0, this);
	connect(modifyArrayPolarAction, SIGNAL(activated()), this, SLOT(onModifyArrayPolar()));
	modifyRotateAction = new QAction(QIconSet(QPixmap::fromMimeSource("rotate.png")), tr("Rotate"), 0, this);
	connect(modifyRotateAction, SIGNAL(activated()), this,
		SLOT(onModifyRotate()));
	modifyStretchAction = new QAction(QIconSet(QPixmap::fromMimeSource("stretch.png")), tr("Stretch"), 0, this);
	connect(modifyStretchAction, SIGNAL(activated()), this, SLOT(onModifyStretch()));
	modifyScaleAction = new QAction(QIconSet(QPixmap::fromMimeSource("scale.png")), tr("Scale"), 0, this);
	connect(modifyScaleAction, SIGNAL(activated()), this, SLOT(onModifyScale()));
	modifyLengthenAction = new QAction(QIconSet(QPixmap::fromMimeSource("lengthen.png")), tr("Lengthen"), 0, this);
	connect(modifyLengthenAction, SIGNAL(activated()), this, SLOT(onModifyLengthen()));
	modifyTrimAction = new QAction(QIconSet(QPixmap::fromMimeSource("trim.png")), tr("Trim"), 0, this);
	connect(modifyTrimAction, SIGNAL(activated()), this, SLOT(onModifyTrim()));
	modifyExtendAction = new QAction(QIconSet(QPixmap::fromMimeSource("extend.png")), tr("Extend"), 0, this);
	connect(modifyExtendAction, SIGNAL(activated()), this, SLOT(onModifyExtend()));
	modifyBreakAction = new QAction(QIconSet(QPixmap::fromMimeSource("break.png")), tr("Break"), 0, this);
	connect(modifyBreakAction, SIGNAL(activated()), this, SLOT(onModifyBreak()));
	modifyEditTextAction = new QAction(QIconSet(QPixmap::fromMimeSource("ddedit.png")), tr("Edit Text"), 0, this);
	connect(modifyEditTextAction, SIGNAL(activated()), this, SLOT(onModifyEditText()));
	modifyHatchEditAction = new QAction(QIconSet(QPixmap::fromMimeSource("hatchedit.png")), tr("Edit Hatching"), 0, this);
	connect(modifyHatchEditAction, SIGNAL(activated()), this, SLOT(onModifyHatchEdit()));
	modifyChamferAction = new QAction(QIconSet(QPixmap::fromMimeSource("chamfer.png")), tr("Chamfer"), 0, this);
	connect(modifyChamferAction, SIGNAL(activated()), this, SLOT(onModifyChamfer()));
	modifyFilletAction = new QAction(QIconSet(QPixmap::fromMimeSource("fillet.png")), tr("Fillet"), 0, this);
	connect(modifyFilletAction, SIGNAL(activated()), this, SLOT(onModifyFillet()));
	modifyExplodeAction = new QAction(QIconSet(QPixmap::fromMimeSource("explode.png")), tr("Explode"), 0, this);
	connect(modifyExplodeAction, SIGNAL(activated()), this, SLOT(onModifyExplode()));
	modifyEraseAction = new QAction(QIconSet(QPixmap::fromMimeSource("erase.png")), tr("Erase"), 0, this);
	connect(modifyEraseAction, SIGNAL(activated()), this, SLOT(onModifyErase()));
}

void MainWindow::createSnapActions()
{
	snapFromPoint = new QAction(QIconSet(QPixmap::fromMimeSource("from.png")), tr("Snap From Point"), 0, this);
	connect(snapFromPoint, SIGNAL(activated()), this, SLOT(onSnapFromPoint()));
	snapEndpoint = new QAction(QIconSet(QPixmap::fromMimeSource("endp.png")), tr("Snap Endpoint"), 0, this);
	connect(snapEndpoint, SIGNAL(activated()), this, SLOT(onSnapEndpoint()));
	snapMidpoint = new QAction(QIconSet(QPixmap::fromMimeSource("mid.png")), tr("Snap Midpoint"), 0, this);
	connect(snapMidpoint, SIGNAL(activated()), this, SLOT(onSnapMidpoint()));
	snapIntersection = new QAction(QIconSet(QPixmap::fromMimeSource("int.png")), tr("Snap Intersection"), 0, this);
	connect(snapIntersection, SIGNAL(activated()), this, SLOT(onSnapIntersection()));
	snapCenter = new QAction(QIconSet(QPixmap::fromMimeSource("cen.png")), tr("Snap Center"), 0, this);
	connect(snapCenter, SIGNAL(activated()), this, SLOT(onSnapCenter()));
	snapQuadrant = new QAction(QIconSet(QPixmap::fromMimeSource("qua.png")), tr("Snap Quadrant"), 0, this);
	connect(snapQuadrant, SIGNAL(activated()), this, SLOT(onSnapQuadrant()));
	snapPerpendicular = new QAction(QIconSet(QPixmap::fromMimeSource("per.png")), tr("Snap Perpendicular"), 0, this);
	connect(snapPerpendicular, SIGNAL(activated()), this, SLOT(onSnapPerpendicular()));
	snapTangent = new QAction(QIconSet(QPixmap::fromMimeSource("tan.png")), tr("Snap Tangent"), 0, this);
	connect(snapTangent, SIGNAL(activated()), this, SLOT(onSnapTangent()));
	snapNode = new QAction(QIconSet(QPixmap::fromMimeSource("nod.png")), tr("Snap Node"), 0, this);
	connect(snapNode, SIGNAL(activated()), this, SLOT(onSnapNode()));
	snapBasepoint = new QAction(QIconSet(QPixmap::fromMimeSource("ins.png")), tr("Snap Basepoint"), 0, this);
	connect(snapBasepoint, SIGNAL(activated()), this, SLOT(onSnapBasepoint()));
	snapNearest = new QAction(QIconSet(QPixmap::fromMimeSource("nea.png")), tr("Snap Nearest"), 0, this);
	connect(snapNearest, SIGNAL(activated()), this, SLOT(onSnapNearest()));
	snapNon = new QAction(QIconSet(QPixmap::fromMimeSource("non.png")), tr("No Snap"), 0, this);
	connect(snapNon, SIGNAL(activated()), this, SLOT(onSnapNon()));
	snapObject = new QAction(QIconSet(QPixmap::fromMimeSource("osnap.png")), tr("Object Snap"), 0, this);
	connect(snapObject, SIGNAL(activated()), this, SLOT(onSnapObject()));
	snapCalculator = new QAction(QIconSet(QPixmap::fromMimeSource("cal.png")), tr("Calculator"), 0, this);
	connect(snapCalculator, SIGNAL(activated()), this, SLOT(onSnapCalculator()));
}

void MainWindow::createDimActions()
{
	dimLinear = new QAction(QIconSet(QPixmap::fromMimeSource("dimlinear.png")), tr("Linear Dimensioning"), 0, this);
	connect(dimLinear, SIGNAL(activated()), this, SLOT(onDimLinear()));
	dimAligned = new QAction(QIconSet(QPixmap::fromMimeSource("dimaligned.png")), tr("Aligned Dimensioning"), 0, this);
	connect(dimAligned, SIGNAL(activated()), this, SLOT(onDimAligned()));
	dimRadius = new QAction(QIconSet(QPixmap::fromMimeSource("dimradius.png")), tr("Radius Dimensioning"), 0, this);
	connect(dimRadius, SIGNAL(activated()), this, SLOT(onDimRadius()));
	dimDiameter = new QAction(QIconSet(QPixmap::fromMimeSource("dimdiameter.png")), tr("Diameter Dimensioning"), 0, this);
	connect(dimDiameter, SIGNAL(activated()), this, SLOT(onDimDiameter()));
	dimAngular = new QAction(QIconSet(QPixmap::fromMimeSource("dimangular.png")), tr("Angular Dimensioning"), 0, this);
	connect(dimAngular, SIGNAL(activated()), this, SLOT(onDimAngular()));
	dimOrdinate = new QAction(QIconSet(QPixmap::fromMimeSource("dimordinate.png")), tr("Ordinate Dimensioning"), 0, this);
	connect(dimOrdinate, SIGNAL(activated()), this, SLOT(onDimOrdinate()));
	dimOrdinateX = new QAction(QIconSet(QPixmap::fromMimeSource("dimordinatex.png")), tr("X Data"), 0, this);
	connect(dimOrdinateX, SIGNAL(activated()), this, SLOT(onDimOrdinateX()));
	dimOrdinateY = new QAction(QIconSet(QPixmap::fromMimeSource("dimordinatey.png")), tr("Y Data"), 0, this);
	connect(dimOrdinateY, SIGNAL(activated()), this, SLOT(onDimOrdinateY()));
	dimBaseline = new QAction(QIconSet(QPixmap::fromMimeSource("dimbaseline.png")), tr("Baseline Dimensioning"), 0, this);
	connect(dimBaseline, SIGNAL(activated()), this, SLOT(onDimBaseline()));
	dimContinue = new QAction(QIconSet(QPixmap::fromMimeSource("dimcont.png")), tr("Continue Dimensioning"), 0, this);
	connect(dimContinue, SIGNAL(activated()), this, SLOT(onDimContinue()));
	dimCenter = new QAction(QIconSet(QPixmap::fromMimeSource("dimcenter.png")), tr("Center Dimensioning"), 0, this);
	connect(dimCenter, SIGNAL(activated()), this, SLOT(onDimCenter()));
	dimLeader = new QAction(QIconSet(QPixmap::fromMimeSource("leader.png")), tr("Leader"), 0, this);
	connect(dimLeader, SIGNAL(activated()), this, SLOT(onDimLeader()));
	dimTEdit = new QAction(QIconSet(QPixmap::fromMimeSource("dimtedit.png")), tr("Edit Dimensioning Text"), 0, this);
	connect(dimTEdit, SIGNAL(activated()), this, SLOT(onDimTEdit()));
	dimTERotate = new QAction(QIconSet(QPixmap::fromMimeSource("dimterotate.png")), tr("Rotate"), 0, this);
	connect(dimTERotate, SIGNAL(activated()), this, SLOT(onDimTERotate()));
	dimTELeft = new QAction(QIconSet(QPixmap::fromMimeSource("dimteleft.png")), tr("Left"), 0, this);
	connect(dimTELeft, SIGNAL(activated()), this, SLOT(onDimTELeft()));
	dimTEHome = new QAction(QIconSet(QPixmap::fromMimeSource("dimtehome.png")), tr("Home"), 0, this);
	connect(dimTEHome, SIGNAL(activated()), this, SLOT(onDimTEHome()));
	dimTERight = new QAction(QIconSet(QPixmap::fromMimeSource("dimteright.png")), tr("Right"), 0, this);
	connect(dimTERight, SIGNAL(activated()), this, SLOT(onDimTERight()));
	dimStyle = new QAction(QIconSet(QPixmap::fromMimeSource("ddim.png")), tr("Dimensioning Style"), 0, this);
	connect(dimStyle, SIGNAL(activated()), this, SLOT(onDimStyle()));
	dimEdit = new QAction(QIconSet(QPixmap::fromMimeSource("dimedit.png")), tr("Edit Dimensioning"), 0, this);
	connect(dimEdit, SIGNAL(activated()), this, SLOT(onDimEdit()));
}

/***************************************************************************
 * Public Slots                                                            *
 ***************************************************************************/

void MainWindow::onFileNew()
{
	commandHistory->add("NEW");
	drawingManager->fileNew(this);
	setIOState(IO_Ready);
}

void MainWindow::onFileOpen()
{
	commandHistory->add("OPEN");
	drawingManager->fileOpen(this);
	entityView->refresh(drawingManager->getDrawing());
	setIOState(IO_Ready);
}

void MainWindow::onFileSave()
{
	commandHistory->add("SAVE");
	drawingManager->fileSave(this);
	setIOState(IO_Ready);
}

void MainWindow::onFileSaveAs()
{
	commandHistory->add("SAVEAS");
	drawingManager->fileSaveAs(this);
}

void MainWindow::onFilePlot()
{
	commandHistory->add("PLOT");
	drawingManager->filePlot(this);
}

void MainWindow::onFileQuit()
{
	quitApplication();
}

void MainWindow::onEditUndo()
{
	commandHistory->add("UNDO");
	drawingManager->editUndo(this);
}

void MainWindow::onEditRedo()
{
	commandHistory->add("REDO");
	drawingManager->editRedo(this);
}

void MainWindow::onEditCut()
{
	drawingManager->editCut(this);
}
void MainWindow::onEditCopy()
{
	drawingManager->editCopy(this);
}

void MainWindow::onEditPaste()
{
	drawingManager->editPaste(this);
}

void MainWindow::onToolsOptions()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onToolsOptions()");
}

void MainWindow::onToolsEntityView()
{
	moveDockWindow(entityView, DockRight);
}

void MainWindow::onHelpHelp()
{

}

void MainWindow::onHelpAbout()
{
	AboutDialog dlg(this);
	dlg.exec();
}

void MainWindow::onPropertiesLayer()
{
	commandHistory->add("LAYER");
	drawingManager->propertiesLayer(this);
}

void MainWindow::onPropertiesLayerChange(const QString& string)
{
	drawingManager->propertiesLayerChange(string, this);
}

void MainWindow::onPropertiesColor()
{
	drawingManager->propertiesColor(this);
}

void MainWindow::onPropertiesLinetype()
{
	drawingManager->propertiesLinetype(this);
}

void MainWindow::onPropertiesLinetypeChange(const QString& string)
{
	drawingManager->propertiesLinetypeChange(string, this);
}

void MainWindow::onPropertiesObject()
{
	drawingManager->propertiesObject(this);
}

void MainWindow::onPropertiesProperties()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onPropertiesProperties()");
}
		
void MainWindow::onZoomIn()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onZoomIn()");
}

void MainWindow::onZoomOut()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onZoomOut()");
}

void MainWindow::onZoomWindow()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onZoomWindow()");
}

void MainWindow::onZoomAll()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onZoomAll()");
}

void MainWindow::onZoomPrevious()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onZoomPrevious()");
}

void MainWindow::onZoomScale()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onZoomScale()");
}

void MainWindow::onZoomExtent()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onZoomExtent()");
}

void MainWindow::onZoomCenter()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onZoomCenter()");
}

void MainWindow::onDrawLine()
{
	commandHistory->add("LINE");
	drawingManager->drawLine(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawXLine()
{
	commandHistory->add("XLINE");
	drawingManager->drawXLine(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawRay()
{
	commandHistory->add("RAY");
	drawingManager->drawRay(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawPLine()
{
	commandHistory->add("PLINE");
	drawingManager->drawPLine(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawMLine()
{
	commandHistory->add("MLINE");
	drawingManager->drawMLine(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawArc()
{
	commandHistory->add("ARC");
	drawingManager->drawArc(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawArcContinue()
{
	drawingManager->drawArcContinue(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawCircle()
{
	commandHistory->add("CIRCLE");
	drawingManager->drawCircle(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawRing()
{
	commandHistory->add("DONUT");
	drawingManager->drawRing(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawEllipse()
{
	commandHistory->add("ELLIPSE");
	drawingManager->drawEllipse(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawPolygon()
{
	commandHistory->add("POLYGON");
	drawingManager->drawPolygon(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawRectangle()
{
	commandHistory->add("RECTANG");
	drawingManager->drawRectangle(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawSolid()
{
	commandHistory->add("SOLID");
	drawingManager->drawSolid(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawPoint()
{
	commandHistory->add("POINT");
	drawingManager->drawPoint(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawBlock()
{
	commandHistory->add("BLOCK");
	drawingManager->drawBlock(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawBlockInsert()
{
	commandHistory->add("INSERT");
	drawingManager->drawBlockInsert(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawHatch()
{
	commandHistory->add("HATCH");
	drawingManager->drawHatch(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawText()
{
	commandHistory->add("MTEXT");
	drawingManager->drawText(this);
	setIOState(IO_Ready);
}

void MainWindow::onDrawTextSL()
{
	drawingManager->drawTextSL(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyMove()
{
	commandHistory->add("MOVE");
	drawingManager->modifyMove(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyCopy()
{
	commandHistory->add("COPY");
	drawingManager->modifyCopy(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyOffset()
{
	commandHistory->add("OFFSET");
	drawingManager->modifyOffset(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyMirror()
{
	commandHistory->add("MIRROR");
	drawingManager->modifyMirror(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyArrayRect()
{
	commandHistory->add("ARRAY");
	drawingManager->modifyArrayRect(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyArrayPolar()
{
	drawingManager->modifyArrayPolar(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyRotate()
{
	commandHistory->add("ROTATE");
	drawingManager->modifyRotate(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyStretch()
{
	commandHistory->add("STRETCH");
	drawingManager->modifyStretch(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyScale()
{
	commandHistory->add("SCALE");
	drawingManager->modifyScale(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyLengthen()
{
	commandHistory->add("LENGTHEN");
	drawingManager->modifyLengthen(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyTrim()
{
	commandHistory->add("TRIM");
	drawingManager->modifyTrim(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyExtend()
{
	commandHistory->add("EXTEND");
	drawingManager->modifyExtend(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyBreak()
{
	commandHistory->add("BREAK");
	drawingManager->modifyBreak(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyEditText()
{
	commandHistory->add("DDEDIT");
	drawingManager->modifyEditText(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyHatchEdit()
{
	commandHistory->add("HATCHEDIT");
	drawingManager->modifyHatchEdit(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyChamfer()
{
	commandHistory->add("CHAMFER");
	drawingManager->modifyChamfer(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyFillet()
{
	commandHistory->add("FILLET");
	drawingManager->modifyFillet(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyExplode()
{
	commandHistory->add("EXPLODE");
	drawingManager->modifyExplode(this);
	setIOState(IO_Ready);
}

void MainWindow::onModifyErase()
{
	drawingManager->modifyErase(this);
	commandHistory->add("ERASE");
	setIOState(IO_Ready);
}

void MainWindow::onSnapFromPoint()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapFromPoint()");
}

void MainWindow::onSnapEndpoint()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapEndpoint()");
}

void MainWindow::onSnapMidpoint()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapMidpoint()");
}

void MainWindow::onSnapIntersection()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapIntersection()");
}

void MainWindow::onSnapCenter()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapCenter()");
}

void MainWindow::onSnapQuadrant()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapQuadrant()");
}

void MainWindow::onSnapPerpendicular()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapPerpendicular()");
}

void MainWindow::onSnapTangent()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapTangent()");
}

void MainWindow::onSnapNode()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapNode()");
}

void MainWindow::onSnapBasepoint()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapBasepoint()");
}

void MainWindow::onSnapNearest()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapNearest()");
}

void MainWindow::onSnapNon()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapNon()");
}

void MainWindow::onSnapObject()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapObject()");
}

void MainWindow::onSnapCalculator()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onSnapCalculator()");
}

void MainWindow::onDimLinear()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimLinear()");
}

void MainWindow::onDimAligned()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimAligned()");
}

void MainWindow::onDimRadius()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimRadius()");
}

void MainWindow::onDimDiameter()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimDiamter()");
}

void MainWindow::onDimAngular()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimAngular()");
}

void MainWindow::onDimOrdinate()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimOrdinate()");
}

void MainWindow::onDimOrdinateX()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimOrdinateX()");
}

void MainWindow::onDimOrdinateY()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimOrdinateY()");
}

void MainWindow::onDimBaseline()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimBaseline()");
}

void MainWindow::onDimContinue()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimContinue()");
}

void MainWindow::onDimCenter()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimCenter()");
}

void MainWindow::onDimLeader()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimLeader()");
}

void MainWindow::onDimTEdit()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimTEdit()");
}

void MainWindow::onDimTERotate()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimTERotate()");
}

void MainWindow::onDimTELeft()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimTELeft()");
}

void MainWindow::onDimTEHome()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimTEHome()");
}

void MainWindow::onDimTERight()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimTERight()");
}

void MainWindow::onDimStyle()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimStyle()");
}

void MainWindow::onDimEdit()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::onDimEdit()");
}

void MainWindow::commandInput(QString& command)
{
	if (!locked)
	{
		runCommand(command);
		commandHistory->reset();
		setIOState(IO_Ready);
	}
	else
	{
		setIOState(IO_Finished);
		lastInput = command;
	}
}

void MainWindow::escape()
{
	if (getIOState() != IO_Ready)
	{
		setIOState(IO_Cancelled);
		printLine("*"+tr("Cancel")+"*");
	}
}

// EOF
