/***************************************************************************
 *                                                                         *
 *   src/concadio/src/creators/arccreator.cpp                              *
 *   (C) 2005 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <concadio/creators/arccreator.h>

#include <concadio/iointerface.h>
#include <concadio/drawingmanager.h>
#include <cad/engine/caddrawing.h>
#include <cad/engine/cadarc.h>
#include <cad/engine/cadvector.h>

#include <qobject.h>

#include <vector>
#include <cmath>

using namespace concadio;
using namespace concadio::creators;
using namespace cad::engine;
using namespace std;

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/** Constructor */
ArcCreator::ArcCreator(DrawingManager* dm) : EntityCreator(dm)
{

}

/** Destructor */
ArcCreator::~ArcCreator()
{

}

/***************************************************************************
 * Public Methods                                                          *
 ***************************************************************************/

/**
 * Start the creation process
 */
void ArcCreator::create(IOInterface* io)
{
	QString input;
	try
	{
		CADVector startPoint, endPoint;
		QStringList options;
		options << "c";
		input = readPointOptionInput(io, QObject::tr("Specify start point for arc or [Center]: "), options);
		CADArc newarc;
		if (input == "c")
		{
			// get center
			input = readPointInput(io, QObject::tr("Specify center point for arc: "));
			CADVector center = CADVector::parseString(input);
			if (!center.isAbsolute()) center = center + io->getLastPoint();
			io->setLastPoint(newarc.getCenter());
			newarc.setCenter(center);

			// get start point
			input = readPointInput(io, QObject::tr("Specify start point for arc: "));
			startPoint = CADVector::parseString(input);
			if (!startPoint.isAbsolute()) startPoint = startPoint + io->getLastPoint();
			io->setLastPoint(startPoint);
			newarc.setRadius((startPoint - newarc.getCenter()).getLength());
			newarc.setStartAngle((startPoint - newarc.getCenter()).getAngle());
			
			options.clear();
			options << "a";
			input = readPointOptionInput(io, QObject::tr("Specify end point for arc or [Angle]: "), options);
			if (input == "a")
			{
				newarc.setAngle(readAngleInput(io, QObject::tr("Specify included angle: "), 
					newarc.getCenter(), newarc.getStartPoint()));
			}
			else
			{
				endPoint = CADVector::parseString(input);
				if (!endPoint.isAbsolute()) endPoint = endPoint + io->getLastPoint();
				io->setLastPoint(endPoint);
				CADVector v = endPoint - newarc.getCenter();
				newarc.setEndAngle((endPoint - newarc.getCenter()).getAngle());
			}
		}
		else
		{
			startPoint = CADVector::parseString(input);
			if (!startPoint.isAbsolute()) startPoint = startPoint + io->getLastPoint();
			io->setLastPoint(startPoint);
			options.clear();
			options << "c" << "e";
			input = readPointOptionInput(io, QObject::tr("Specify second point for arc or [Center/Endpoint]: "), options);
			
			if (input == "c")
			{
				input = readPointInput(io, QObject::tr("Specify center point for arc: "));
				CADVector center = CADVector::parseString(input);
				if (!center.isAbsolute()) center = center + io->getLastPoint();
				io->setLastPoint(center);
				newarc.setCenter(center);				
				newarc.setStartAngle((startPoint-newarc.getCenter()).getAngle());
				newarc.setRadius((startPoint-newarc.getCenter()).getLength());
				options.clear();
				options << "a";
				input = readPointOptionInput(io, QObject::tr("Specify end point for arc or [Angle]: "), options);
				if (input == "a")
				{
					newarc.setAngle(readAngleInput(io, QObject::tr("Specify included angle: "), 
						newarc.getCenter(), newarc.getStartPoint()));
				}
				else
				{
					endPoint = CADVector::parseString(input);
					
					newarc.setEndAngle((endPoint - newarc.getCenter()).getAngle());
				}
			}
			else if (input == "e")
			{
				input = readPointInput(io, QObject::tr("Specify end point for arc: "));
				endPoint = CADVector::parseString(input);
				if (!endPoint.isAbsolute()) endPoint = endPoint + io->getLastPoint();
				io->setLastPoint(endPoint);
				
				QStringList options;
				options << "a" << "r";
				input = readPointOptionInput(io, QObject::tr("Specify center point for arc or [Angle/Radius]: "), options);
				
				if (input == "a")
				{
					double angle = readValueInput(io, QObject::tr("Specify included angle: "));
					newarc.setStartEndPointAngle(startPoint, endPoint, angle);
				}
				else if (input == "r")
				{
					double radius = readValueInput(io, QObject::tr("Specify radius for arc: "));
					newarc.setStartEndPointRadius(startPoint, endPoint, radius);
				}
				else
				{
					CADVector center = CADVector::parseString(input);
					if (!center.isAbsolute()) center = center + io->getLastPoint();
					io->setLastPoint(center);
					newarc.setCenter(center);
					newarc.setStartAngle((startPoint - newarc.getCenter()).getAngle());
					newarc.setEndAngle((endPoint - newarc.getCenter()).getAngle());
					newarc.setRadius((startPoint - newarc.getCenter()).getLength());
				}
			}
			else
			{
				CADVector second = CADVector::parseString(input);
				if (!second.isAbsolute()) second = second + io->getLastPoint();
				io->setLastPoint(second);
				
				input = readPointInput(io, QObject::tr("Specify end point for arc: "));
				endPoint = CADVector::parseString(input);
				if (!endPoint.isAbsolute()) endPoint = endPoint + io->getLastPoint();
				io->setLastPoint(endPoint);
				newarc.set3Points(startPoint, second, endPoint);
			}
		}
		if (newarc.getRadius() <= 0.0)
		{
			io->printLine(QObject::tr("Invalid."));
			return;
		}
		CADArc* arc = new CADArc(newarc.getCenter(), newarc.getRadius(), newarc.getStartAngle(), 
			newarc.getEndAngle(), dm->getDrawing()->getActiveLayer().getName(), 
			dm->getDrawing()->getActivePen());
		dm->getDrawing()->addEntity(arc);
		undoAction = new UndoNewArc(*arc);
		io->draw(dm->getDrawing());
	}
	catch (...)
	{
		return;
	}
}

/**
 * undo the last creation part
 */
void ArcCreator::undo(IOInterface* io)
{

}

UndoAble* ArcCreator::getUndoActions()
{
	return undoAction;
}

// EOF
