/***************************************************************************
 *                                                                         *
 *   src/concadio/src/creators/circlecreator.cpp                           *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <concadio/creators/circlecreator.h>

#include <concadio/iointerface.h>
#include <concadio/drawingmanager.h>
#include <cad/engine/caddrawing.h>
#include <cad/engine/cadcircle.h>
#include <cad/engine/cadvector.h>

#include <qobject.h>
#include <vector>
#include <cmath>

using namespace concadio;
using namespace concadio::creators;
using namespace cad::engine;
using namespace std;

double CircleCreator::lastRadius = 0.0;

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/** Constructor */
CircleCreator::CircleCreator(DrawingManager* dm) : EntityCreator(dm)
{

}

/** Destructor */
CircleCreator::~CircleCreator()
{

}

/***************************************************************************
 * Public Methods                                                          *
 ***************************************************************************/

/**
 * Start the creation process
 */
void CircleCreator::create(IOInterface* io)
{
	try
	{
		QString input;
		bool valid = false;
		QStringList options;
		options << "2p";
		input = readPointOptionInput(io, QObject::tr("Specify center point for circle or [2P]: "), options);
		
		CADVector center;
		double radius = 0.0;
		// 2 point mode selected?
		if (input == "2p")
		{
			CADVector p1, p2;
			for (int i = 0; i<2; i++)
			{
				QString input = readPointInput(io, QString(QObject::tr("Specify %1 end point of circle\'s diameter: ").arg(i ? "second" : "first")));
				if (i)
				{
					p2 = CADVector::parseString(input);
					if (!p2.isAbsolute()) p2 = io->getLastPoint() + p2;
					io->setLastPoint(p2);
				}
				else
				{
					p1 = CADVector::parseString(input);
					if (!p1.isAbsolute()) p1 = io->getLastPoint() + p1;
					io->setLastPoint(p1);
				}
			}
			// Calculate center and radius
			radius = fabs((p1-p2).getLength())/2.0;
			center.setX((p2-p1).getX()*-1.0/2.0);
			center.setY((p2-p1).getY()*-1.0/2.0);
			center = p2 + center;
	}
	else
	{
		valid = false;
		center = CADVector::parseString(input);
		if (!center.isAbsolute()) center = io->getLastPoint() + center;
		io->setLastPoint(center);
		bool diameterInput = false;
		do
		{
			if (!diameterInput)
			{
				QString s;
				s.sprintf("%1.4f", lastRadius);
				io->print(QObject::tr("Specify radius for circle or [Diameter]")
				+(lastRadius == 0.0 ? "" : " <"+s+">")+": ");
			}
			else
			{
				QString s;
				s.sprintf("%1.4f", lastRadius/2.0);
				io->print(QObject::tr("Specify diameter for circle")+
				(lastRadius == 0.0 ? "" : " <"+s+">")+": ");
			}
			try
			{
				input = io->readInput();
				if (input == "d")
				{
					diameterInput = true;
					continue;
				}
				if ((lastRadius != 0.0) && (input == ""))
				{
					radius = lastRadius;
					valid = true;
				}
				else
				{
					radius = input.toDouble(&valid);
					if (!valid)
					{
						CADVector point = CADVector::parseString(input);
						radius = fabs((point-center).getLength());
						valid = true;
					}
				}
				if ((!valid) || (radius <= 0.0))
					io->printLine(QObject::tr("Invalid."));
				if (diameterInput)
					radius /= 2.0;
			}
			catch (Exception e)
			{
				if (io->getIOState() == IO_Cancelled) return;
				if (diameterInput) io->printLine(e.getError());
				else io->printLine(QObject::tr("Invalid."));
			}
		}
		while (!valid);
	}
	if (radius <= 0.0)
	{
		io->printLine(QObject::tr("Invalid."));
		return;
	}
	// Create circle
	CADCircle* circle = new CADCircle(center, radius, 
		dm->getDrawing()->getActiveLayer().getName(), dm->getDrawing()->getActivePen());
	dm->getDrawing()->addEntity(circle);
	undoAction = new UndoNewCircle(*circle);
	io->draw(dm->getDrawing());
	lastRadius = radius;
	}
	catch (...)
	{
		return;
	}
}

/**
 * undo the last creation part
 */
void CircleCreator::undo(IOInterface* io)
{

}

UndoAble* CircleCreator::getUndoActions()
{
	return undoAction;
}

// EOF
