/***************************************************************************
 *                                                                         *
 *   src/concadio/src/creators/entitycreator.cpp                           *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <concadio/creators/entitycreator.h>

#include <concadio/iointerface.h>
#include <concadio/drawingmanager.h>

#include <qobject.h>
#include <cmath>

using namespace concadio;
using namespace concadio::creators;

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/** Constructor */
EntityCreator::EntityCreator(DrawingManager* dm)
{
	this->dm = dm;
}

/** Destructor */
EntityCreator::~EntityCreator()
{

}

/***************************************************************************
 * Protected Methods                                                       *
 ***************************************************************************/

/**
 * Read a point or option from the command line
 * @param options: accept only the given options
 */
QString EntityCreator::readPointOptionInput(IOInterface* io, QString prompt, QStringList options) throw (Exception)
{
	QString input;
	bool valid = false;
	do
	{
		io->print(prompt);
		input = io->readInput();
		valid = options.contains(input);
		if (!valid)
		{
			try
			{
				CADVector::parseString(input);
				valid = true;
			}
			catch (Exception e)
			{
				if (io->getIOState() == IO_Cancelled) throw e;
				io->printLine(QObject::tr("Point or option keyword required."));
			}
		}
	}
	while (!valid);
	return input;
}

/**
 * Read a point from the command line
 */
QString EntityCreator::readPointInput(IOInterface* io, QString prompt) throw (Exception)
{
	QString input;
	bool valid = false;
	do
	{
		io->print(prompt);
		input = io->readInput();
		try
		{
			CADVector().parseString(input);
			valid = true;
		}
		catch (Exception e)
		{
			if (io->getIOState() == IO_Cancelled) throw e;
			io->printLine(QObject::tr("Invalid."));
		}
	}
	while (!valid);
	return input;
}

/**
 * Read a numeric value from the command line
 */
double EntityCreator::readValueInput(IOInterface* io, QString prompt) throw (Exception)
{
	double value = 0.0;
	bool valid = false;
	QString input;
	do
	{
		io->print(prompt);
		try
		{
			input = io->readInput();
			value = input.toDouble(&valid);
		}
		catch (Exception e)
		{
			if (io->getIOState() == IO_Cancelled) throw e;
			else io->printLine(QObject::tr("Invalid."));
		}
	}
	while (!valid);
	return value;
}

/**
 * Read a numeric value from the command line
 * @param point: if a point is entered the return value is the distance 
 * between the given point and the entered point
 */
double EntityCreator::readValueInput(IOInterface* io, QString prompt, CADVector point) throw (Exception)
{
	double value = 0.0;
	bool valid = false;
	QString input;
	do
	{
		io->print(prompt);
		try
		{
			input = io->readInput();
			value = input.toDouble(&valid);
			if (valid) break;
			CADVector point2 = CADVector::parseString(input);
			if (!point2.isAbsolute()) point2 = point2 + io->getLastPoint();
			io->setLastPoint(point2);
			valid = true;
			value = (point2 - point).getLength();
		}
		catch (Exception e)
		{
			if (io->getIOState() == IO_Cancelled) throw e;
			else io->printLine(QObject::tr("Invalid."));
		}
	}
	while (!valid);
	return value;
}

/**
 * Read an angle from the command line
 * @param def: default value, if no default leave QString::null
 */
double EntityCreator::readAngleInput(IOInterface* io, QString prompt, QString def) throw (Exception)
{
	bool valid = false;
	double angle = 0.0;
	do
	{
		io->print(prompt);
		try
		{
			QString input = io->readInput();
			if ((input == "") && (def != QString::null))
				input = def;
			angle = input.toDouble(&valid);
			if (!valid) io->printLine(QObject::tr("Invalid."));
		}
		catch (Exception e)
		{
			if (io->getIOState() == IO_Cancelled) throw e;
			io->printLine(e.getError());
		}
	}
	while (!valid);
	while (fabs(angle) >= 360) angle -= (angle > 0 ? 360.0 : -360.0);
	return angle;
}

/**
 * Read an angle from the command line or accept a point which includes 
 * an angle with the given center and start point
 * @param def: default value, if no default leave QString::null
 */
double EntityCreator::readAngleInput(IOInterface* io, QString prompt, CADVector center, 
	CADVector start, QString def) throw (Exception)
{
	double angle = 0.0;
	bool valid = false;
	do
	{
		io->print(prompt);
		try
		{
			QString input = io->readInput();
			if ((input == "") && (def != QString::null))
				input = def;
			angle = input.toDouble(&valid);
			if (valid) break;
			
			CADVector point = CADVector::parseString(input);
			if (!point.isAbsolute()) point = point + io->getLastPoint();
			io->setLastPoint(point);
			valid = true;
			CADVector va = start - center;
			CADVector vb = point - center;
			angle = acos((va*vb)/(va.getLength()*vb.getLength()))*(180/(4.0*atan(1.0)));
		}
		catch (Exception e)
		{
			if (io->getIOState() == IO_Cancelled) throw e;
			io->printLine(QObject::tr("Invalid."));
		}
	}
	while (!valid);
	return angle;
}

// EOF
