/***************************************************************************
 *                                                                         *
 *   src/concadio/src/drawingmanager.cpp                                   *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <concadio/drawingmanager.h>

#include <concadio/iointerface.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qfile.h>

#include <cad/engine/cadline.h>
#include <cad/engine/cadcircle.h>
#include <cad/engine/cadarc.h>
#include <cad/engine/cadellipse.h>
#include <cad/engine/cadpoint.h>
#include <cad/engine/cadxline.h>
#include <cad/engine/cadray.h>
#include <cad/xmlexport/xmldrawing.h>

#include <concadio/undoredo/undo.h>
#include <concadio/undoredo/undoactivepen.h>
#include <concadio/undoredo/undolayer.h>
#include <concadio/undoredo/undolinetype.h>

#include <concadio/creators/arccreator.h>
#include <concadio/creators/circlecreator.h>
#include <concadio/creators/ellipsecreator.h>
#include <concadio/creators/linecreator.h>
#include <concadio/creators/pointcreator.h>
#include <concadio/creators/raycreator.h>
#include <concadio/creators/xlinecreator.h>

using namespace concadio;
using namespace concadio::creators;
using namespace concadio::undoredo;
using namespace cad::engine;
using cad::xmlexport::XMLDrawing;

DrawingManager* DrawingManager::instance = 0;

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

// Private Constructor
DrawingManager::DrawingManager()
{
	drawing = new CADDrawing;
	undoHistory = new UndoHistory;
}

DrawingManager::~DrawingManager()
{
	
}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

DrawingManager* DrawingManager::getInstance()
{
	if (instance == 0)
	{
		instance = new DrawingManager();
	}
	return instance;
}

/** @return the drawing */
CADDrawing* DrawingManager::getDrawing() const
{
	return drawing;
}

/** @return the undo history */
UndoHistory* DrawingManager::getUndoHistory() const
{
	return undoHistory;
}

/***************************************************************************
 * Public Methods                                                          *
 ***************************************************************************/

/** read a drawing from the given file */
CADDrawing DrawingManager::readFile(QString fileName) throw (Exception)
{
	QDomDocument doc;
	QFile file(fileName);
	if (!file.open(IO_ReadOnly))
	{
		throw Exception(QObject::tr("Cannot open file %1 for reading.").arg(fileName));
	}
	doc.setContent(&file);
	file.close();
	XMLDrawing x;
	CADDrawing drawing = XMLDrawing::xmlImport(doc.documentElement());
	drawing.setFileName(fileName);
	return drawing;
}

/** write the given drawing to a file */
void DrawingManager::writeFile(CADDrawing drawing) throw (Exception)
{
	XMLDrawing xmlDrawing;
	QDomDocument doc = xmlDrawing.xmlExport(drawing);
	QFile file (drawing.getFileName());
	if (file.open(IO_WriteOnly))
	{
		QTextStream stream(&file);
		stream << "<?xml version=\"1.0\"?>\n";
		doc.save(stream, 2);
		file.close();
	}
	else
		throw Exception(QObject::tr("Cannot open file %1 for writing.").arg(drawing.getFileName()));
}

void DrawingManager::fileNew(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		// TODO: Ask if user wants to save changes on current drawing
		CADDrawing drawing;
		if (iointerface->getNewDrawing(drawing))
		{
			try
			{
				this->writeFile(drawing);
				delete this->drawing;
				this->drawing = new CADDrawing(drawing);
				iointerface->updatePropertiesDisplay();
				undoHistory->clear();
			}
			catch (Exception e)
			{
				QMessageBox::critical(0, QObject::tr("Error"), e.getError());
			}
		}
	}
}

void DrawingManager::fileOpen(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		// TODO: Ask if user wants to save changes on current drawing
		QString path = QFileDialog::getOpenFileName("~/", "ConstructCAD Drawings (*.cdw);;All Files (*)");
		if (!path.isEmpty())
		{
			try
			{
				CADDrawing newDrawing = readFile(path);
				delete drawing;
				drawing = new CADDrawing(newDrawing);
				iointerface->updatePropertiesDisplay();
				undoHistory->clear();
			}
			catch (Exception e)
			{
				QMessageBox::critical(0, QObject::tr("Error"), e.getError());
			}
		}
	}
}

void DrawingManager::fileSave(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		if (drawing->getFileName() == QString::null)
			fileSaveAs(iointerface);
		else
		{
			writeFile(*drawing);
			drawing->setModified(false);
			iointerface->updatePropertiesDisplay();
		}
	}
}

void DrawingManager::fileSaveAs(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QString path = QFileDialog::getSaveFileName("~/", "ConstructCAD Drawings (*.cdw);;All Files (*)");
		if (!path.isEmpty())
		{
			drawing->setFileName(path);
			try
			{
				writeFile(*drawing);
				drawing->setModified(false);
				iointerface->updatePropertiesDisplay();
			}
			catch (Exception e)
			{
				QMessageBox::critical(0, QObject::tr("Error"), e.getError());
			}
		}
	}
}

void DrawingManager::filePlot(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::filePlot()");
	}
}

void DrawingManager::editUndo(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
	if (undoHistory->undoPossible())
		undoHistory->undo(this, iointerface);
	else
		QMessageBox::information(0, QObject::tr("Error"), QObject::tr("Undo is not possible."));
	}
	else
	{
		// TODO: undo parts of the running command if possible
	}
}

void DrawingManager::editRedo(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		if (undoHistory->redoPossible())
			undoHistory->redo(this, iointerface);
		else
			QMessageBox::information(0, QObject::tr("Error"), QObject::tr("Redo is not possible."));
	}
}

void DrawingManager::editCut(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::editCut()");
	}
}

void DrawingManager::editCopy(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::editCopy()");
	}
}

void DrawingManager::editPaste(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::editPaste()");
	}
}

void DrawingManager::propertiesLayer(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		CADLayerList layers = drawing->getLayerList();
		LayerRenamingManager rm;
		UndoAble* undoAble;
		if (iointerface->getLayerChanges(layers, rm, undoAble))
		{
			drawing->updateLayerList(layers, rm);
			undoHistory->addUndoAble(undoAble);
			iointerface->updatePropertiesDisplay();
		}
	}
}

void DrawingManager::propertiesLayerChange(const QString& string, IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		UndoActiveLayer* undo = new UndoActiveLayer(drawing->getActiveLayer().getName(), string);
		drawing->setActiveLayer(string);
		undoHistory->addUndoAble(undo);
		iointerface->updatePropertiesDisplay();
	}
}

void DrawingManager::propertiesColor(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		CADColor color = drawing->getActivePen().getColor();
		if (iointerface->getColor(color, drawing->getActiveLayer().getName()))
		{
			CADPen pen = drawing->getActivePen();
			pen.setColor(color);
			UndoActivePen* undo = new UndoActivePen(drawing->getActivePen(), pen);
			drawing->setActivePen(pen);
			undoHistory->addUndoAble(undo);
			iointerface->updatePropertiesDisplay();
		}
	}
}

void DrawingManager::propertiesLinetype(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		CADPen pen = drawing->getActivePen();
		QString linetype = pen.getLinetype();
		CADLinetypeList linetypelist = drawing->getLinetypeList();
		UndoAbleGroup* undoAble;
		if (iointerface->getLinetype(linetype, linetypelist, true, undoAble))
		{
			pen.setLinetype(linetype);
			UndoActivePen* undo = new UndoActivePen(drawing->getActivePen(), pen);
			drawing->setActivePen(pen);
			drawing->updateLinetypeList(linetypelist);
			UndoAbleGroup* undoGroup = new UndoAbleGroup(*undoAble);
			undoGroup->addUndoAble(undo);
			undoHistory->addUndoAble(undoGroup);
			iointerface->updatePropertiesDisplay();
		}
	}
}

void DrawingManager::propertiesLinetypeChange(const QString& string, IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		CADPen pen = drawing->getActivePen();
		pen.setLinetype(string);
		UndoActivePen* undo = new UndoActivePen(drawing->getActivePen(), pen);
		drawing->setActivePen(pen);
		undoHistory->addUndoAble(undo);
		iointerface->updatePropertiesDisplay();
	}
}

void DrawingManager::propertiesObject(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		if (iointerface->getActiveProperties())
		{
			iointerface->updatePropertiesDisplay();
		}
	}
}

void DrawingManager::propertiesProperties(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::propertiesProperties()");
	}
}

void DrawingManager::drawLine(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		iointerface->lock();
		LineCreator creator(this);
		creator.create(iointerface);
		undoHistory->addUndoAble(creator.getUndoActions());
		iointerface->unlock();
	}
}

void DrawingManager::drawXLine(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		iointerface->lock();
		XLineCreator creator(this);
		creator.create(iointerface);
		undoHistory->addUndoAble(creator.getUndoActions());
		iointerface->unlock();
	}
}

void DrawingManager::drawRay(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		iointerface->lock();
		RayCreator creator(this);
		creator.create(iointerface);
		undoHistory->addUndoAble(creator.getUndoActions());
		iointerface->unlock();
	}
}

void DrawingManager::drawPLine(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawPLine()");
	}
}

void DrawingManager::drawMLine(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawMLine()");
	}
}

void DrawingManager::drawArc(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		iointerface->lock();
		ArcCreator creator(this);
		creator.create(iointerface);
		undoHistory->addUndoAble(creator.getUndoActions());
		iointerface->unlock();
	}
}

void DrawingManager::drawArcContinue(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawArcContinue()");
	}
}

void DrawingManager::drawCircle(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		iointerface->lock();
		CircleCreator creator(this);
		creator.create(iointerface);
		undoHistory->addUndoAble(creator.getUndoActions());
		iointerface->unlock();
	}
}

void DrawingManager::drawRing(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawRing()");
	}
}

void DrawingManager::drawEllipse(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		iointerface->lock();
		EllipseCreator creator(this);
		creator.create(iointerface);
		undoHistory->addUndoAble(creator.getUndoActions());
		iointerface->unlock();
	}
}

void DrawingManager::drawPolygon(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawPolygon()");
	}
}

void DrawingManager::drawRectangle(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawRectangle()");
	}
}

void DrawingManager::drawSolid(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawSolid()");
	}
}

void DrawingManager::drawPoint(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		iointerface->lock();
		PointCreator creator(this);
		creator.create(iointerface);
		undoHistory->addUndoAble(creator.getUndoActions());
		iointerface->unlock();
	}
}

void DrawingManager::drawBlock(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawBlock()");
	}
}

void DrawingManager::drawBlockInsert(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawBlockInsert()");
	}
}

void DrawingManager::drawHatch(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawHatch()");
	}
}

void DrawingManager::drawText(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawText()");
	}
}
void DrawingManager::drawTextSL(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::drawPLine()");
	}
}

void DrawingManager::modifyMove(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyMove()");
	}
}

void DrawingManager::modifyCopy(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyCopy()");
	}
}

void DrawingManager::modifyOffset(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyOffset()");
	}
}

void DrawingManager::modifyMirror(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyMirror()");
	}
}

void DrawingManager::modifyArrayRect(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyArrayRect()");
	}
}

void DrawingManager::modifyArrayPolar(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyArrayPolar()");
	}
}

void DrawingManager::modifyRotate(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyRotate()");
	}
}

void DrawingManager::modifyStretch(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyStretch()");
	}
}

void DrawingManager::modifyScale(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyScale()");
	}
}

void DrawingManager::modifyLengthen(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyLengthen()");
	}
}

void DrawingManager::modifyTrim(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyTrim()");
	}
}

void DrawingManager::modifyExtend(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyExtend()");
	}
}

void DrawingManager::modifyBreak(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyBreak()");
	}
}

void DrawingManager::modifyEditText(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyEditText()");
	}
}

void DrawingManager::modifyHatchEdit(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyHatchEdit()");
	}
}

void DrawingManager::modifyChamfer(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyChamfer()");
	}
}

void DrawingManager::modifyFillet(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyFillet()");
	}
}

void DrawingManager::modifyExplode(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyExplode()");
	}
}

void DrawingManager::modifyErase(IOInterface* iointerface)
{
	if (!iointerface->isLocked())
	{
		QMessageBox::information(0, "TODO", "TODO: DrawingManager::modifyErase()");
	}
}

// EOF
